from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class CompressibilityUnits(Enum):
        """
            CompressibilityUnits enumeration
        """
        
        InversePascal = 'InversePascal'
        """
            
        """
        
        InverseKilopascal = 'InverseKilopascal'
        """
            
        """
        
        InverseMegapascal = 'InverseMegapascal'
        """
            
        """
        
        InverseAtmosphere = 'InverseAtmosphere'
        """
            
        """
        
        InverseMillibar = 'InverseMillibar'
        """
            
        """
        
        InverseBar = 'InverseBar'
        """
            
        """
        
        InversePoundForcePerSquareInch = 'InversePoundForcePerSquareInch'
        """
            
        """
        

class CompressibilityDto:
    """
    A DTO representation of a Compressibility

    Attributes:
        value (float): The value of the Compressibility.
        unit (CompressibilityUnits): The specific unit that the Compressibility value is representing.
    """

    def __init__(self, value: float, unit: CompressibilityUnits):
        """
        Create a new DTO representation of a Compressibility

        Parameters:
            value (float): The value of the Compressibility.
            unit (CompressibilityUnits): The specific unit that the Compressibility value is representing.
        """
        self.value: float = value
        """
        The value of the Compressibility
        """
        self.unit: CompressibilityUnits = unit
        """
        The specific unit that the Compressibility value is representing
        """

    def to_json(self):
        """
        Get a Compressibility DTO JSON object representing the current unit.

        :return: JSON object represents Compressibility DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "InversePascal"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of Compressibility DTO from a json representation.

        :param data: The Compressibility DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "InversePascal"}
        :return: A new instance of CompressibilityDto.
        :rtype: CompressibilityDto
        """
        return CompressibilityDto(value=data["value"], unit=CompressibilityUnits(data["unit"]))


class Compressibility(AbstractMeasure):
    """
    None

    Args:
        value (float): The value.
        from_unit (CompressibilityUnits): The Compressibility unit to create from, The default unit is InversePascal
    """
    def __init__(self, value: float, from_unit: CompressibilityUnits = CompressibilityUnits.InversePascal):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__inverse_pascals = None
        
        self.__inverse_kilopascals = None
        
        self.__inverse_megapascals = None
        
        self.__inverse_atmospheres = None
        
        self.__inverse_millibars = None
        
        self.__inverse_bars = None
        
        self.__inverse_pounds_force_per_square_inch = None
        

    def convert(self, unit: CompressibilityUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: CompressibilityUnits = CompressibilityUnits.InversePascal) -> CompressibilityDto:
        """
        Get a new instance of Compressibility DTO representing the current unit.

        :param hold_in_unit: The specific Compressibility unit to store the Compressibility value in the DTO representation.
        :type hold_in_unit: CompressibilityUnits
        :return: A new instance of CompressibilityDto.
        :rtype: CompressibilityDto
        """
        return CompressibilityDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: CompressibilityUnits = CompressibilityUnits.InversePascal):
        """
        Get a Compressibility DTO JSON object representing the current unit.

        :param hold_in_unit: The specific Compressibility unit to store the Compressibility value in the DTO representation.
        :type hold_in_unit: CompressibilityUnits
        :return: JSON object represents Compressibility DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "InversePascal"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(compressibility_dto: CompressibilityDto):
        """
        Obtain a new instance of Compressibility from a DTO unit object.

        :param compressibility_dto: The Compressibility DTO representation.
        :type compressibility_dto: CompressibilityDto
        :return: A new instance of Compressibility.
        :rtype: Compressibility
        """
        return Compressibility(compressibility_dto.value, compressibility_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of Compressibility from a DTO unit json representation.

        :param data: The Compressibility DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "InversePascal"}
        :return: A new instance of Compressibility.
        :rtype: Compressibility
        """
        return Compressibility.from_dto(CompressibilityDto.from_json(data))

    def __convert_from_base(self, from_unit: CompressibilityUnits) -> float:
        value = self._value
        
        if from_unit == CompressibilityUnits.InversePascal:
            return (value)
        
        if from_unit == CompressibilityUnits.InverseKilopascal:
            return (value / 1e3)
        
        if from_unit == CompressibilityUnits.InverseMegapascal:
            return (value / 1e6)
        
        if from_unit == CompressibilityUnits.InverseAtmosphere:
            return (value / 101325)
        
        if from_unit == CompressibilityUnits.InverseMillibar:
            return (value / 100)
        
        if from_unit == CompressibilityUnits.InverseBar:
            return (value / 1e5)
        
        if from_unit == CompressibilityUnits.InversePoundForcePerSquareInch:
            return (value / 6.894757293168361e3)
        
        return None


    def __convert_to_base(self, value: float, to_unit: CompressibilityUnits) -> float:
        
        if to_unit == CompressibilityUnits.InversePascal:
            return (value)
        
        if to_unit == CompressibilityUnits.InverseKilopascal:
            return (value * 1e3)
        
        if to_unit == CompressibilityUnits.InverseMegapascal:
            return (value * 1e6)
        
        if to_unit == CompressibilityUnits.InverseAtmosphere:
            return (value * 101325)
        
        if to_unit == CompressibilityUnits.InverseMillibar:
            return (value * 100)
        
        if to_unit == CompressibilityUnits.InverseBar:
            return (value * 1e5)
        
        if to_unit == CompressibilityUnits.InversePoundForcePerSquareInch:
            return (value * 6.894757293168361e3)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_inverse_pascals(inverse_pascals: float):
        """
        Create a new instance of Compressibility from a value in inverse_pascals.

        

        :param meters: The Compressibility value in inverse_pascals.
        :type inverse_pascals: float
        :return: A new instance of Compressibility.
        :rtype: Compressibility
        """
        return Compressibility(inverse_pascals, CompressibilityUnits.InversePascal)

    
    @staticmethod
    def from_inverse_kilopascals(inverse_kilopascals: float):
        """
        Create a new instance of Compressibility from a value in inverse_kilopascals.

        

        :param meters: The Compressibility value in inverse_kilopascals.
        :type inverse_kilopascals: float
        :return: A new instance of Compressibility.
        :rtype: Compressibility
        """
        return Compressibility(inverse_kilopascals, CompressibilityUnits.InverseKilopascal)

    
    @staticmethod
    def from_inverse_megapascals(inverse_megapascals: float):
        """
        Create a new instance of Compressibility from a value in inverse_megapascals.

        

        :param meters: The Compressibility value in inverse_megapascals.
        :type inverse_megapascals: float
        :return: A new instance of Compressibility.
        :rtype: Compressibility
        """
        return Compressibility(inverse_megapascals, CompressibilityUnits.InverseMegapascal)

    
    @staticmethod
    def from_inverse_atmospheres(inverse_atmospheres: float):
        """
        Create a new instance of Compressibility from a value in inverse_atmospheres.

        

        :param meters: The Compressibility value in inverse_atmospheres.
        :type inverse_atmospheres: float
        :return: A new instance of Compressibility.
        :rtype: Compressibility
        """
        return Compressibility(inverse_atmospheres, CompressibilityUnits.InverseAtmosphere)

    
    @staticmethod
    def from_inverse_millibars(inverse_millibars: float):
        """
        Create a new instance of Compressibility from a value in inverse_millibars.

        

        :param meters: The Compressibility value in inverse_millibars.
        :type inverse_millibars: float
        :return: A new instance of Compressibility.
        :rtype: Compressibility
        """
        return Compressibility(inverse_millibars, CompressibilityUnits.InverseMillibar)

    
    @staticmethod
    def from_inverse_bars(inverse_bars: float):
        """
        Create a new instance of Compressibility from a value in inverse_bars.

        

        :param meters: The Compressibility value in inverse_bars.
        :type inverse_bars: float
        :return: A new instance of Compressibility.
        :rtype: Compressibility
        """
        return Compressibility(inverse_bars, CompressibilityUnits.InverseBar)

    
    @staticmethod
    def from_inverse_pounds_force_per_square_inch(inverse_pounds_force_per_square_inch: float):
        """
        Create a new instance of Compressibility from a value in inverse_pounds_force_per_square_inch.

        

        :param meters: The Compressibility value in inverse_pounds_force_per_square_inch.
        :type inverse_pounds_force_per_square_inch: float
        :return: A new instance of Compressibility.
        :rtype: Compressibility
        """
        return Compressibility(inverse_pounds_force_per_square_inch, CompressibilityUnits.InversePoundForcePerSquareInch)

    
    @property
    def inverse_pascals(self) -> float:
        """
        
        """
        if self.__inverse_pascals != None:
            return self.__inverse_pascals
        self.__inverse_pascals = self.__convert_from_base(CompressibilityUnits.InversePascal)
        return self.__inverse_pascals

    
    @property
    def inverse_kilopascals(self) -> float:
        """
        
        """
        if self.__inverse_kilopascals != None:
            return self.__inverse_kilopascals
        self.__inverse_kilopascals = self.__convert_from_base(CompressibilityUnits.InverseKilopascal)
        return self.__inverse_kilopascals

    
    @property
    def inverse_megapascals(self) -> float:
        """
        
        """
        if self.__inverse_megapascals != None:
            return self.__inverse_megapascals
        self.__inverse_megapascals = self.__convert_from_base(CompressibilityUnits.InverseMegapascal)
        return self.__inverse_megapascals

    
    @property
    def inverse_atmospheres(self) -> float:
        """
        
        """
        if self.__inverse_atmospheres != None:
            return self.__inverse_atmospheres
        self.__inverse_atmospheres = self.__convert_from_base(CompressibilityUnits.InverseAtmosphere)
        return self.__inverse_atmospheres

    
    @property
    def inverse_millibars(self) -> float:
        """
        
        """
        if self.__inverse_millibars != None:
            return self.__inverse_millibars
        self.__inverse_millibars = self.__convert_from_base(CompressibilityUnits.InverseMillibar)
        return self.__inverse_millibars

    
    @property
    def inverse_bars(self) -> float:
        """
        
        """
        if self.__inverse_bars != None:
            return self.__inverse_bars
        self.__inverse_bars = self.__convert_from_base(CompressibilityUnits.InverseBar)
        return self.__inverse_bars

    
    @property
    def inverse_pounds_force_per_square_inch(self) -> float:
        """
        
        """
        if self.__inverse_pounds_force_per_square_inch != None:
            return self.__inverse_pounds_force_per_square_inch
        self.__inverse_pounds_force_per_square_inch = self.__convert_from_base(CompressibilityUnits.InversePoundForcePerSquareInch)
        return self.__inverse_pounds_force_per_square_inch

    
    def to_string(self, unit: CompressibilityUnits = CompressibilityUnits.InversePascal, fractional_digits: int = None) -> str:
        """
        Format the Compressibility to a string.
        
        Note: the default format for Compressibility is InversePascal.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the Compressibility. Default is 'InversePascal'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == CompressibilityUnits.InversePascal:
            return f"""{super()._truncate_fraction_digits(self.inverse_pascals, fractional_digits)} Pa⁻¹"""
        
        if unit == CompressibilityUnits.InverseKilopascal:
            return f"""{super()._truncate_fraction_digits(self.inverse_kilopascals, fractional_digits)} kPa⁻¹"""
        
        if unit == CompressibilityUnits.InverseMegapascal:
            return f"""{super()._truncate_fraction_digits(self.inverse_megapascals, fractional_digits)} MPa⁻¹"""
        
        if unit == CompressibilityUnits.InverseAtmosphere:
            return f"""{super()._truncate_fraction_digits(self.inverse_atmospheres, fractional_digits)} atm⁻¹"""
        
        if unit == CompressibilityUnits.InverseMillibar:
            return f"""{super()._truncate_fraction_digits(self.inverse_millibars, fractional_digits)} mbar⁻¹"""
        
        if unit == CompressibilityUnits.InverseBar:
            return f"""{super()._truncate_fraction_digits(self.inverse_bars, fractional_digits)} bar⁻¹"""
        
        if unit == CompressibilityUnits.InversePoundForcePerSquareInch:
            return f"""{super()._truncate_fraction_digits(self.inverse_pounds_force_per_square_inch, fractional_digits)} psi⁻¹"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: CompressibilityUnits = CompressibilityUnits.InversePascal) -> str:
        """
        Get Compressibility unit abbreviation.
        Note! the default abbreviation for Compressibility is InversePascal.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == CompressibilityUnits.InversePascal:
            return """Pa⁻¹"""
        
        if unit_abbreviation == CompressibilityUnits.InverseKilopascal:
            return """kPa⁻¹"""
        
        if unit_abbreviation == CompressibilityUnits.InverseMegapascal:
            return """MPa⁻¹"""
        
        if unit_abbreviation == CompressibilityUnits.InverseAtmosphere:
            return """atm⁻¹"""
        
        if unit_abbreviation == CompressibilityUnits.InverseMillibar:
            return """mbar⁻¹"""
        
        if unit_abbreviation == CompressibilityUnits.InverseBar:
            return """bar⁻¹"""
        
        if unit_abbreviation == CompressibilityUnits.InversePoundForcePerSquareInch:
            return """psi⁻¹"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class InformationUnits(Enum):
        """
            InformationUnits enumeration
        """
        
        Byte = 'Byte'
        """
            
        """
        
        Bit = 'Bit'
        """
            
        """
        
        Kilobyte = 'Kilobyte'
        """
            
        """
        
        Megabyte = 'Megabyte'
        """
            
        """
        
        Gigabyte = 'Gigabyte'
        """
            
        """
        
        Terabyte = 'Terabyte'
        """
            
        """
        
        Petabyte = 'Petabyte'
        """
            
        """
        
        Exabyte = 'Exabyte'
        """
            
        """
        
        Kibibyte = 'Kibibyte'
        """
            
        """
        
        Mebibyte = 'Mebibyte'
        """
            
        """
        
        Gibibyte = 'Gibibyte'
        """
            
        """
        
        Tebibyte = 'Tebibyte'
        """
            
        """
        
        Pebibyte = 'Pebibyte'
        """
            
        """
        
        Exbibyte = 'Exbibyte'
        """
            
        """
        
        Kilobit = 'Kilobit'
        """
            
        """
        
        Megabit = 'Megabit'
        """
            
        """
        
        Gigabit = 'Gigabit'
        """
            
        """
        
        Terabit = 'Terabit'
        """
            
        """
        
        Petabit = 'Petabit'
        """
            
        """
        
        Exabit = 'Exabit'
        """
            
        """
        
        Kibibit = 'Kibibit'
        """
            
        """
        
        Mebibit = 'Mebibit'
        """
            
        """
        
        Gibibit = 'Gibibit'
        """
            
        """
        
        Tebibit = 'Tebibit'
        """
            
        """
        
        Pebibit = 'Pebibit'
        """
            
        """
        
        Exbibit = 'Exbibit'
        """
            
        """
        

class InformationDto:
    """
    A DTO representation of a Information

    Attributes:
        value (float): The value of the Information.
        unit (InformationUnits): The specific unit that the Information value is representing.
    """

    def __init__(self, value: float, unit: InformationUnits):
        """
        Create a new DTO representation of a Information

        Parameters:
            value (float): The value of the Information.
            unit (InformationUnits): The specific unit that the Information value is representing.
        """
        self.value: float = value
        """
        The value of the Information
        """
        self.unit: InformationUnits = unit
        """
        The specific unit that the Information value is representing
        """

    def to_json(self):
        """
        Get a Information DTO JSON object representing the current unit.

        :return: JSON object represents Information DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "Bit"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of Information DTO from a json representation.

        :param data: The Information DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "Bit"}
        :return: A new instance of InformationDto.
        :rtype: InformationDto
        """
        return InformationDto(value=data["value"], unit=InformationUnits(data["unit"]))


class Information(AbstractMeasure):
    """
    In computing and telecommunications, a unit of information is the capacity of some standard data storage system or communication channel, used to measure the capacities of other systems and channels. In information theory, units of information are also used to measure the information contents or entropy of random variables.

    Args:
        value (float): The value.
        from_unit (InformationUnits): The Information unit to create from, The default unit is Bit
    """
    def __init__(self, value: float, from_unit: InformationUnits = InformationUnits.Bit):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__bytes = None
        
        self.__bits = None
        
        self.__kilobytes = None
        
        self.__megabytes = None
        
        self.__gigabytes = None
        
        self.__terabytes = None
        
        self.__petabytes = None
        
        self.__exabytes = None
        
        self.__kibibytes = None
        
        self.__mebibytes = None
        
        self.__gibibytes = None
        
        self.__tebibytes = None
        
        self.__pebibytes = None
        
        self.__exbibytes = None
        
        self.__kilobits = None
        
        self.__megabits = None
        
        self.__gigabits = None
        
        self.__terabits = None
        
        self.__petabits = None
        
        self.__exabits = None
        
        self.__kibibits = None
        
        self.__mebibits = None
        
        self.__gibibits = None
        
        self.__tebibits = None
        
        self.__pebibits = None
        
        self.__exbibits = None
        

    def convert(self, unit: InformationUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: InformationUnits = InformationUnits.Bit) -> InformationDto:
        """
        Get a new instance of Information DTO representing the current unit.

        :param hold_in_unit: The specific Information unit to store the Information value in the DTO representation.
        :type hold_in_unit: InformationUnits
        :return: A new instance of InformationDto.
        :rtype: InformationDto
        """
        return InformationDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: InformationUnits = InformationUnits.Bit):
        """
        Get a Information DTO JSON object representing the current unit.

        :param hold_in_unit: The specific Information unit to store the Information value in the DTO representation.
        :type hold_in_unit: InformationUnits
        :return: JSON object represents Information DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "Bit"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(information_dto: InformationDto):
        """
        Obtain a new instance of Information from a DTO unit object.

        :param information_dto: The Information DTO representation.
        :type information_dto: InformationDto
        :return: A new instance of Information.
        :rtype: Information
        """
        return Information(information_dto.value, information_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of Information from a DTO unit json representation.

        :param data: The Information DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "Bit"}
        :return: A new instance of Information.
        :rtype: Information
        """
        return Information.from_dto(InformationDto.from_json(data))

    def __convert_from_base(self, from_unit: InformationUnits) -> float:
        value = self._value
        
        if from_unit == InformationUnits.Byte:
            return (value / 8)
        
        if from_unit == InformationUnits.Bit:
            return (value)
        
        if from_unit == InformationUnits.Kilobyte:
            return ((value / 8) / 1000.0)
        
        if from_unit == InformationUnits.Megabyte:
            return ((value / 8) / 1000000.0)
        
        if from_unit == InformationUnits.Gigabyte:
            return ((value / 8) / 1000000000.0)
        
        if from_unit == InformationUnits.Terabyte:
            return ((value / 8) / 1000000000000.0)
        
        if from_unit == InformationUnits.Petabyte:
            return ((value / 8) / 1000000000000000.0)
        
        if from_unit == InformationUnits.Exabyte:
            return ((value / 8) / 1e+18)
        
        if from_unit == InformationUnits.Kibibyte:
            return ((value / 8) / 1024)
        
        if from_unit == InformationUnits.Mebibyte:
            return ((value / 8) / 1048576)
        
        if from_unit == InformationUnits.Gibibyte:
            return ((value / 8) / 1073741824)
        
        if from_unit == InformationUnits.Tebibyte:
            return ((value / 8) / 1099511627776)
        
        if from_unit == InformationUnits.Pebibyte:
            return ((value / 8) / 1125899906842624)
        
        if from_unit == InformationUnits.Exbibyte:
            return ((value / 8) / 1152921504606846976)
        
        if from_unit == InformationUnits.Kilobit:
            return ((value) / 1000.0)
        
        if from_unit == InformationUnits.Megabit:
            return ((value) / 1000000.0)
        
        if from_unit == InformationUnits.Gigabit:
            return ((value) / 1000000000.0)
        
        if from_unit == InformationUnits.Terabit:
            return ((value) / 1000000000000.0)
        
        if from_unit == InformationUnits.Petabit:
            return ((value) / 1000000000000000.0)
        
        if from_unit == InformationUnits.Exabit:
            return ((value) / 1e+18)
        
        if from_unit == InformationUnits.Kibibit:
            return ((value) / 1024)
        
        if from_unit == InformationUnits.Mebibit:
            return ((value) / 1048576)
        
        if from_unit == InformationUnits.Gibibit:
            return ((value) / 1073741824)
        
        if from_unit == InformationUnits.Tebibit:
            return ((value) / 1099511627776)
        
        if from_unit == InformationUnits.Pebibit:
            return ((value) / 1125899906842624)
        
        if from_unit == InformationUnits.Exbibit:
            return ((value) / 1152921504606846976)
        
        return None


    def __convert_to_base(self, value: float, to_unit: InformationUnits) -> float:
        
        if to_unit == InformationUnits.Byte:
            return (value * 8)
        
        if to_unit == InformationUnits.Bit:
            return (value)
        
        if to_unit == InformationUnits.Kilobyte:
            return ((value * 8) * 1000.0)
        
        if to_unit == InformationUnits.Megabyte:
            return ((value * 8) * 1000000.0)
        
        if to_unit == InformationUnits.Gigabyte:
            return ((value * 8) * 1000000000.0)
        
        if to_unit == InformationUnits.Terabyte:
            return ((value * 8) * 1000000000000.0)
        
        if to_unit == InformationUnits.Petabyte:
            return ((value * 8) * 1000000000000000.0)
        
        if to_unit == InformationUnits.Exabyte:
            return ((value * 8) * 1e+18)
        
        if to_unit == InformationUnits.Kibibyte:
            return ((value * 8) * 1024)
        
        if to_unit == InformationUnits.Mebibyte:
            return ((value * 8) * 1048576)
        
        if to_unit == InformationUnits.Gibibyte:
            return ((value * 8) * 1073741824)
        
        if to_unit == InformationUnits.Tebibyte:
            return ((value * 8) * 1099511627776)
        
        if to_unit == InformationUnits.Pebibyte:
            return ((value * 8) * 1125899906842624)
        
        if to_unit == InformationUnits.Exbibyte:
            return ((value * 8) * 1152921504606846976)
        
        if to_unit == InformationUnits.Kilobit:
            return ((value) * 1000.0)
        
        if to_unit == InformationUnits.Megabit:
            return ((value) * 1000000.0)
        
        if to_unit == InformationUnits.Gigabit:
            return ((value) * 1000000000.0)
        
        if to_unit == InformationUnits.Terabit:
            return ((value) * 1000000000000.0)
        
        if to_unit == InformationUnits.Petabit:
            return ((value) * 1000000000000000.0)
        
        if to_unit == InformationUnits.Exabit:
            return ((value) * 1e+18)
        
        if to_unit == InformationUnits.Kibibit:
            return ((value) * 1024)
        
        if to_unit == InformationUnits.Mebibit:
            return ((value) * 1048576)
        
        if to_unit == InformationUnits.Gibibit:
            return ((value) * 1073741824)
        
        if to_unit == InformationUnits.Tebibit:
            return ((value) * 1099511627776)
        
        if to_unit == InformationUnits.Pebibit:
            return ((value) * 1125899906842624)
        
        if to_unit == InformationUnits.Exbibit:
            return ((value) * 1152921504606846976)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_bytes(bytes: float):
        """
        Create a new instance of Information from a value in bytes.

        

        :param meters: The Information value in bytes.
        :type bytes: float
        :return: A new instance of Information.
        :rtype: Information
        """
        return Information(bytes, InformationUnits.Byte)

    
    @staticmethod
    def from_bits(bits: float):
        """
        Create a new instance of Information from a value in bits.

        

        :param meters: The Information value in bits.
        :type bits: float
        :return: A new instance of Information.
        :rtype: Information
        """
        return Information(bits, InformationUnits.Bit)

    
    @staticmethod
    def from_kilobytes(kilobytes: float):
        """
        Create a new instance of Information from a value in kilobytes.

        

        :param meters: The Information value in kilobytes.
        :type kilobytes: float
        :return: A new instance of Information.
        :rtype: Information
        """
        return Information(kilobytes, InformationUnits.Kilobyte)

    
    @staticmethod
    def from_megabytes(megabytes: float):
        """
        Create a new instance of Information from a value in megabytes.

        

        :param meters: The Information value in megabytes.
        :type megabytes: float
        :return: A new instance of Information.
        :rtype: Information
        """
        return Information(megabytes, InformationUnits.Megabyte)

    
    @staticmethod
    def from_gigabytes(gigabytes: float):
        """
        Create a new instance of Information from a value in gigabytes.

        

        :param meters: The Information value in gigabytes.
        :type gigabytes: float
        :return: A new instance of Information.
        :rtype: Information
        """
        return Information(gigabytes, InformationUnits.Gigabyte)

    
    @staticmethod
    def from_terabytes(terabytes: float):
        """
        Create a new instance of Information from a value in terabytes.

        

        :param meters: The Information value in terabytes.
        :type terabytes: float
        :return: A new instance of Information.
        :rtype: Information
        """
        return Information(terabytes, InformationUnits.Terabyte)

    
    @staticmethod
    def from_petabytes(petabytes: float):
        """
        Create a new instance of Information from a value in petabytes.

        

        :param meters: The Information value in petabytes.
        :type petabytes: float
        :return: A new instance of Information.
        :rtype: Information
        """
        return Information(petabytes, InformationUnits.Petabyte)

    
    @staticmethod
    def from_exabytes(exabytes: float):
        """
        Create a new instance of Information from a value in exabytes.

        

        :param meters: The Information value in exabytes.
        :type exabytes: float
        :return: A new instance of Information.
        :rtype: Information
        """
        return Information(exabytes, InformationUnits.Exabyte)

    
    @staticmethod
    def from_kibibytes(kibibytes: float):
        """
        Create a new instance of Information from a value in kibibytes.

        

        :param meters: The Information value in kibibytes.
        :type kibibytes: float
        :return: A new instance of Information.
        :rtype: Information
        """
        return Information(kibibytes, InformationUnits.Kibibyte)

    
    @staticmethod
    def from_mebibytes(mebibytes: float):
        """
        Create a new instance of Information from a value in mebibytes.

        

        :param meters: The Information value in mebibytes.
        :type mebibytes: float
        :return: A new instance of Information.
        :rtype: Information
        """
        return Information(mebibytes, InformationUnits.Mebibyte)

    
    @staticmethod
    def from_gibibytes(gibibytes: float):
        """
        Create a new instance of Information from a value in gibibytes.

        

        :param meters: The Information value in gibibytes.
        :type gibibytes: float
        :return: A new instance of Information.
        :rtype: Information
        """
        return Information(gibibytes, InformationUnits.Gibibyte)

    
    @staticmethod
    def from_tebibytes(tebibytes: float):
        """
        Create a new instance of Information from a value in tebibytes.

        

        :param meters: The Information value in tebibytes.
        :type tebibytes: float
        :return: A new instance of Information.
        :rtype: Information
        """
        return Information(tebibytes, InformationUnits.Tebibyte)

    
    @staticmethod
    def from_pebibytes(pebibytes: float):
        """
        Create a new instance of Information from a value in pebibytes.

        

        :param meters: The Information value in pebibytes.
        :type pebibytes: float
        :return: A new instance of Information.
        :rtype: Information
        """
        return Information(pebibytes, InformationUnits.Pebibyte)

    
    @staticmethod
    def from_exbibytes(exbibytes: float):
        """
        Create a new instance of Information from a value in exbibytes.

        

        :param meters: The Information value in exbibytes.
        :type exbibytes: float
        :return: A new instance of Information.
        :rtype: Information
        """
        return Information(exbibytes, InformationUnits.Exbibyte)

    
    @staticmethod
    def from_kilobits(kilobits: float):
        """
        Create a new instance of Information from a value in kilobits.

        

        :param meters: The Information value in kilobits.
        :type kilobits: float
        :return: A new instance of Information.
        :rtype: Information
        """
        return Information(kilobits, InformationUnits.Kilobit)

    
    @staticmethod
    def from_megabits(megabits: float):
        """
        Create a new instance of Information from a value in megabits.

        

        :param meters: The Information value in megabits.
        :type megabits: float
        :return: A new instance of Information.
        :rtype: Information
        """
        return Information(megabits, InformationUnits.Megabit)

    
    @staticmethod
    def from_gigabits(gigabits: float):
        """
        Create a new instance of Information from a value in gigabits.

        

        :param meters: The Information value in gigabits.
        :type gigabits: float
        :return: A new instance of Information.
        :rtype: Information
        """
        return Information(gigabits, InformationUnits.Gigabit)

    
    @staticmethod
    def from_terabits(terabits: float):
        """
        Create a new instance of Information from a value in terabits.

        

        :param meters: The Information value in terabits.
        :type terabits: float
        :return: A new instance of Information.
        :rtype: Information
        """
        return Information(terabits, InformationUnits.Terabit)

    
    @staticmethod
    def from_petabits(petabits: float):
        """
        Create a new instance of Information from a value in petabits.

        

        :param meters: The Information value in petabits.
        :type petabits: float
        :return: A new instance of Information.
        :rtype: Information
        """
        return Information(petabits, InformationUnits.Petabit)

    
    @staticmethod
    def from_exabits(exabits: float):
        """
        Create a new instance of Information from a value in exabits.

        

        :param meters: The Information value in exabits.
        :type exabits: float
        :return: A new instance of Information.
        :rtype: Information
        """
        return Information(exabits, InformationUnits.Exabit)

    
    @staticmethod
    def from_kibibits(kibibits: float):
        """
        Create a new instance of Information from a value in kibibits.

        

        :param meters: The Information value in kibibits.
        :type kibibits: float
        :return: A new instance of Information.
        :rtype: Information
        """
        return Information(kibibits, InformationUnits.Kibibit)

    
    @staticmethod
    def from_mebibits(mebibits: float):
        """
        Create a new instance of Information from a value in mebibits.

        

        :param meters: The Information value in mebibits.
        :type mebibits: float
        :return: A new instance of Information.
        :rtype: Information
        """
        return Information(mebibits, InformationUnits.Mebibit)

    
    @staticmethod
    def from_gibibits(gibibits: float):
        """
        Create a new instance of Information from a value in gibibits.

        

        :param meters: The Information value in gibibits.
        :type gibibits: float
        :return: A new instance of Information.
        :rtype: Information
        """
        return Information(gibibits, InformationUnits.Gibibit)

    
    @staticmethod
    def from_tebibits(tebibits: float):
        """
        Create a new instance of Information from a value in tebibits.

        

        :param meters: The Information value in tebibits.
        :type tebibits: float
        :return: A new instance of Information.
        :rtype: Information
        """
        return Information(tebibits, InformationUnits.Tebibit)

    
    @staticmethod
    def from_pebibits(pebibits: float):
        """
        Create a new instance of Information from a value in pebibits.

        

        :param meters: The Information value in pebibits.
        :type pebibits: float
        :return: A new instance of Information.
        :rtype: Information
        """
        return Information(pebibits, InformationUnits.Pebibit)

    
    @staticmethod
    def from_exbibits(exbibits: float):
        """
        Create a new instance of Information from a value in exbibits.

        

        :param meters: The Information value in exbibits.
        :type exbibits: float
        :return: A new instance of Information.
        :rtype: Information
        """
        return Information(exbibits, InformationUnits.Exbibit)

    
    @property
    def bytes(self) -> float:
        """
        
        """
        if self.__bytes != None:
            return self.__bytes
        self.__bytes = self.__convert_from_base(InformationUnits.Byte)
        return self.__bytes

    
    @property
    def bits(self) -> float:
        """
        
        """
        if self.__bits != None:
            return self.__bits
        self.__bits = self.__convert_from_base(InformationUnits.Bit)
        return self.__bits

    
    @property
    def kilobytes(self) -> float:
        """
        
        """
        if self.__kilobytes != None:
            return self.__kilobytes
        self.__kilobytes = self.__convert_from_base(InformationUnits.Kilobyte)
        return self.__kilobytes

    
    @property
    def megabytes(self) -> float:
        """
        
        """
        if self.__megabytes != None:
            return self.__megabytes
        self.__megabytes = self.__convert_from_base(InformationUnits.Megabyte)
        return self.__megabytes

    
    @property
    def gigabytes(self) -> float:
        """
        
        """
        if self.__gigabytes != None:
            return self.__gigabytes
        self.__gigabytes = self.__convert_from_base(InformationUnits.Gigabyte)
        return self.__gigabytes

    
    @property
    def terabytes(self) -> float:
        """
        
        """
        if self.__terabytes != None:
            return self.__terabytes
        self.__terabytes = self.__convert_from_base(InformationUnits.Terabyte)
        return self.__terabytes

    
    @property
    def petabytes(self) -> float:
        """
        
        """
        if self.__petabytes != None:
            return self.__petabytes
        self.__petabytes = self.__convert_from_base(InformationUnits.Petabyte)
        return self.__petabytes

    
    @property
    def exabytes(self) -> float:
        """
        
        """
        if self.__exabytes != None:
            return self.__exabytes
        self.__exabytes = self.__convert_from_base(InformationUnits.Exabyte)
        return self.__exabytes

    
    @property
    def kibibytes(self) -> float:
        """
        
        """
        if self.__kibibytes != None:
            return self.__kibibytes
        self.__kibibytes = self.__convert_from_base(InformationUnits.Kibibyte)
        return self.__kibibytes

    
    @property
    def mebibytes(self) -> float:
        """
        
        """
        if self.__mebibytes != None:
            return self.__mebibytes
        self.__mebibytes = self.__convert_from_base(InformationUnits.Mebibyte)
        return self.__mebibytes

    
    @property
    def gibibytes(self) -> float:
        """
        
        """
        if self.__gibibytes != None:
            return self.__gibibytes
        self.__gibibytes = self.__convert_from_base(InformationUnits.Gibibyte)
        return self.__gibibytes

    
    @property
    def tebibytes(self) -> float:
        """
        
        """
        if self.__tebibytes != None:
            return self.__tebibytes
        self.__tebibytes = self.__convert_from_base(InformationUnits.Tebibyte)
        return self.__tebibytes

    
    @property
    def pebibytes(self) -> float:
        """
        
        """
        if self.__pebibytes != None:
            return self.__pebibytes
        self.__pebibytes = self.__convert_from_base(InformationUnits.Pebibyte)
        return self.__pebibytes

    
    @property
    def exbibytes(self) -> float:
        """
        
        """
        if self.__exbibytes != None:
            return self.__exbibytes
        self.__exbibytes = self.__convert_from_base(InformationUnits.Exbibyte)
        return self.__exbibytes

    
    @property
    def kilobits(self) -> float:
        """
        
        """
        if self.__kilobits != None:
            return self.__kilobits
        self.__kilobits = self.__convert_from_base(InformationUnits.Kilobit)
        return self.__kilobits

    
    @property
    def megabits(self) -> float:
        """
        
        """
        if self.__megabits != None:
            return self.__megabits
        self.__megabits = self.__convert_from_base(InformationUnits.Megabit)
        return self.__megabits

    
    @property
    def gigabits(self) -> float:
        """
        
        """
        if self.__gigabits != None:
            return self.__gigabits
        self.__gigabits = self.__convert_from_base(InformationUnits.Gigabit)
        return self.__gigabits

    
    @property
    def terabits(self) -> float:
        """
        
        """
        if self.__terabits != None:
            return self.__terabits
        self.__terabits = self.__convert_from_base(InformationUnits.Terabit)
        return self.__terabits

    
    @property
    def petabits(self) -> float:
        """
        
        """
        if self.__petabits != None:
            return self.__petabits
        self.__petabits = self.__convert_from_base(InformationUnits.Petabit)
        return self.__petabits

    
    @property
    def exabits(self) -> float:
        """
        
        """
        if self.__exabits != None:
            return self.__exabits
        self.__exabits = self.__convert_from_base(InformationUnits.Exabit)
        return self.__exabits

    
    @property
    def kibibits(self) -> float:
        """
        
        """
        if self.__kibibits != None:
            return self.__kibibits
        self.__kibibits = self.__convert_from_base(InformationUnits.Kibibit)
        return self.__kibibits

    
    @property
    def mebibits(self) -> float:
        """
        
        """
        if self.__mebibits != None:
            return self.__mebibits
        self.__mebibits = self.__convert_from_base(InformationUnits.Mebibit)
        return self.__mebibits

    
    @property
    def gibibits(self) -> float:
        """
        
        """
        if self.__gibibits != None:
            return self.__gibibits
        self.__gibibits = self.__convert_from_base(InformationUnits.Gibibit)
        return self.__gibibits

    
    @property
    def tebibits(self) -> float:
        """
        
        """
        if self.__tebibits != None:
            return self.__tebibits
        self.__tebibits = self.__convert_from_base(InformationUnits.Tebibit)
        return self.__tebibits

    
    @property
    def pebibits(self) -> float:
        """
        
        """
        if self.__pebibits != None:
            return self.__pebibits
        self.__pebibits = self.__convert_from_base(InformationUnits.Pebibit)
        return self.__pebibits

    
    @property
    def exbibits(self) -> float:
        """
        
        """
        if self.__exbibits != None:
            return self.__exbibits
        self.__exbibits = self.__convert_from_base(InformationUnits.Exbibit)
        return self.__exbibits

    
    def to_string(self, unit: InformationUnits = InformationUnits.Bit, fractional_digits: int = None) -> str:
        """
        Format the Information to a string.
        
        Note: the default format for Information is Bit.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the Information. Default is 'Bit'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == InformationUnits.Byte:
            return f"""{super()._truncate_fraction_digits(self.bytes, fractional_digits)} B"""
        
        if unit == InformationUnits.Bit:
            return f"""{super()._truncate_fraction_digits(self.bits, fractional_digits)} b"""
        
        if unit == InformationUnits.Kilobyte:
            return f"""{super()._truncate_fraction_digits(self.kilobytes, fractional_digits)} kB"""
        
        if unit == InformationUnits.Megabyte:
            return f"""{super()._truncate_fraction_digits(self.megabytes, fractional_digits)} MB"""
        
        if unit == InformationUnits.Gigabyte:
            return f"""{super()._truncate_fraction_digits(self.gigabytes, fractional_digits)} GB"""
        
        if unit == InformationUnits.Terabyte:
            return f"""{super()._truncate_fraction_digits(self.terabytes, fractional_digits)} TB"""
        
        if unit == InformationUnits.Petabyte:
            return f"""{super()._truncate_fraction_digits(self.petabytes, fractional_digits)} PB"""
        
        if unit == InformationUnits.Exabyte:
            return f"""{super()._truncate_fraction_digits(self.exabytes, fractional_digits)} EB"""
        
        if unit == InformationUnits.Kibibyte:
            return f"""{super()._truncate_fraction_digits(self.kibibytes, fractional_digits)} KiBB"""
        
        if unit == InformationUnits.Mebibyte:
            return f"""{super()._truncate_fraction_digits(self.mebibytes, fractional_digits)} MiBB"""
        
        if unit == InformationUnits.Gibibyte:
            return f"""{super()._truncate_fraction_digits(self.gibibytes, fractional_digits)} GiBB"""
        
        if unit == InformationUnits.Tebibyte:
            return f"""{super()._truncate_fraction_digits(self.tebibytes, fractional_digits)} TiBB"""
        
        if unit == InformationUnits.Pebibyte:
            return f"""{super()._truncate_fraction_digits(self.pebibytes, fractional_digits)} PiBB"""
        
        if unit == InformationUnits.Exbibyte:
            return f"""{super()._truncate_fraction_digits(self.exbibytes, fractional_digits)} EiBB"""
        
        if unit == InformationUnits.Kilobit:
            return f"""{super()._truncate_fraction_digits(self.kilobits, fractional_digits)} kb"""
        
        if unit == InformationUnits.Megabit:
            return f"""{super()._truncate_fraction_digits(self.megabits, fractional_digits)} Mb"""
        
        if unit == InformationUnits.Gigabit:
            return f"""{super()._truncate_fraction_digits(self.gigabits, fractional_digits)} Gb"""
        
        if unit == InformationUnits.Terabit:
            return f"""{super()._truncate_fraction_digits(self.terabits, fractional_digits)} Tb"""
        
        if unit == InformationUnits.Petabit:
            return f"""{super()._truncate_fraction_digits(self.petabits, fractional_digits)} Pb"""
        
        if unit == InformationUnits.Exabit:
            return f"""{super()._truncate_fraction_digits(self.exabits, fractional_digits)} Eb"""
        
        if unit == InformationUnits.Kibibit:
            return f"""{super()._truncate_fraction_digits(self.kibibits, fractional_digits)} KiBb"""
        
        if unit == InformationUnits.Mebibit:
            return f"""{super()._truncate_fraction_digits(self.mebibits, fractional_digits)} MiBb"""
        
        if unit == InformationUnits.Gibibit:
            return f"""{super()._truncate_fraction_digits(self.gibibits, fractional_digits)} GiBb"""
        
        if unit == InformationUnits.Tebibit:
            return f"""{super()._truncate_fraction_digits(self.tebibits, fractional_digits)} TiBb"""
        
        if unit == InformationUnits.Pebibit:
            return f"""{super()._truncate_fraction_digits(self.pebibits, fractional_digits)} PiBb"""
        
        if unit == InformationUnits.Exbibit:
            return f"""{super()._truncate_fraction_digits(self.exbibits, fractional_digits)} EiBb"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: InformationUnits = InformationUnits.Bit) -> str:
        """
        Get Information unit abbreviation.
        Note! the default abbreviation for Information is Bit.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == InformationUnits.Byte:
            return """B"""
        
        if unit_abbreviation == InformationUnits.Bit:
            return """b"""
        
        if unit_abbreviation == InformationUnits.Kilobyte:
            return """kB"""
        
        if unit_abbreviation == InformationUnits.Megabyte:
            return """MB"""
        
        if unit_abbreviation == InformationUnits.Gigabyte:
            return """GB"""
        
        if unit_abbreviation == InformationUnits.Terabyte:
            return """TB"""
        
        if unit_abbreviation == InformationUnits.Petabyte:
            return """PB"""
        
        if unit_abbreviation == InformationUnits.Exabyte:
            return """EB"""
        
        if unit_abbreviation == InformationUnits.Kibibyte:
            return """KiBB"""
        
        if unit_abbreviation == InformationUnits.Mebibyte:
            return """MiBB"""
        
        if unit_abbreviation == InformationUnits.Gibibyte:
            return """GiBB"""
        
        if unit_abbreviation == InformationUnits.Tebibyte:
            return """TiBB"""
        
        if unit_abbreviation == InformationUnits.Pebibyte:
            return """PiBB"""
        
        if unit_abbreviation == InformationUnits.Exbibyte:
            return """EiBB"""
        
        if unit_abbreviation == InformationUnits.Kilobit:
            return """kb"""
        
        if unit_abbreviation == InformationUnits.Megabit:
            return """Mb"""
        
        if unit_abbreviation == InformationUnits.Gigabit:
            return """Gb"""
        
        if unit_abbreviation == InformationUnits.Terabit:
            return """Tb"""
        
        if unit_abbreviation == InformationUnits.Petabit:
            return """Pb"""
        
        if unit_abbreviation == InformationUnits.Exabit:
            return """Eb"""
        
        if unit_abbreviation == InformationUnits.Kibibit:
            return """KiBb"""
        
        if unit_abbreviation == InformationUnits.Mebibit:
            return """MiBb"""
        
        if unit_abbreviation == InformationUnits.Gibibit:
            return """GiBb"""
        
        if unit_abbreviation == InformationUnits.Tebibit:
            return """TiBb"""
        
        if unit_abbreviation == InformationUnits.Pebibit:
            return """PiBb"""
        
        if unit_abbreviation == InformationUnits.Exbibit:
            return """EiBb"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class AmplitudeRatioUnits(Enum):
        """
            AmplitudeRatioUnits enumeration
        """
        
        DecibelVolt = 'DecibelVolt'
        """
            
        """
        
        DecibelMicrovolt = 'DecibelMicrovolt'
        """
            
        """
        
        DecibelMillivolt = 'DecibelMillivolt'
        """
            
        """
        
        DecibelUnloaded = 'DecibelUnloaded'
        """
            
        """
        

class AmplitudeRatioDto:
    """
    A DTO representation of a AmplitudeRatio

    Attributes:
        value (float): The value of the AmplitudeRatio.
        unit (AmplitudeRatioUnits): The specific unit that the AmplitudeRatio value is representing.
    """

    def __init__(self, value: float, unit: AmplitudeRatioUnits):
        """
        Create a new DTO representation of a AmplitudeRatio

        Parameters:
            value (float): The value of the AmplitudeRatio.
            unit (AmplitudeRatioUnits): The specific unit that the AmplitudeRatio value is representing.
        """
        self.value: float = value
        """
        The value of the AmplitudeRatio
        """
        self.unit: AmplitudeRatioUnits = unit
        """
        The specific unit that the AmplitudeRatio value is representing
        """

    def to_json(self):
        """
        Get a AmplitudeRatio DTO JSON object representing the current unit.

        :return: JSON object represents AmplitudeRatio DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "DecibelVolt"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of AmplitudeRatio DTO from a json representation.

        :param data: The AmplitudeRatio DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "DecibelVolt"}
        :return: A new instance of AmplitudeRatioDto.
        :rtype: AmplitudeRatioDto
        """
        return AmplitudeRatioDto(value=data["value"], unit=AmplitudeRatioUnits(data["unit"]))


class AmplitudeRatio(AbstractMeasure):
    """
    The strength of a signal expressed in decibels (dB) relative to one volt RMS.

    Args:
        value (float): The value.
        from_unit (AmplitudeRatioUnits): The AmplitudeRatio unit to create from, The default unit is DecibelVolt
    """
    def __init__(self, value: float, from_unit: AmplitudeRatioUnits = AmplitudeRatioUnits.DecibelVolt):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__decibel_volts = None
        
        self.__decibel_microvolts = None
        
        self.__decibel_millivolts = None
        
        self.__decibels_unloaded = None
        

    def convert(self, unit: AmplitudeRatioUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: AmplitudeRatioUnits = AmplitudeRatioUnits.DecibelVolt) -> AmplitudeRatioDto:
        """
        Get a new instance of AmplitudeRatio DTO representing the current unit.

        :param hold_in_unit: The specific AmplitudeRatio unit to store the AmplitudeRatio value in the DTO representation.
        :type hold_in_unit: AmplitudeRatioUnits
        :return: A new instance of AmplitudeRatioDto.
        :rtype: AmplitudeRatioDto
        """
        return AmplitudeRatioDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: AmplitudeRatioUnits = AmplitudeRatioUnits.DecibelVolt):
        """
        Get a AmplitudeRatio DTO JSON object representing the current unit.

        :param hold_in_unit: The specific AmplitudeRatio unit to store the AmplitudeRatio value in the DTO representation.
        :type hold_in_unit: AmplitudeRatioUnits
        :return: JSON object represents AmplitudeRatio DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "DecibelVolt"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(amplitude_ratio_dto: AmplitudeRatioDto):
        """
        Obtain a new instance of AmplitudeRatio from a DTO unit object.

        :param amplitude_ratio_dto: The AmplitudeRatio DTO representation.
        :type amplitude_ratio_dto: AmplitudeRatioDto
        :return: A new instance of AmplitudeRatio.
        :rtype: AmplitudeRatio
        """
        return AmplitudeRatio(amplitude_ratio_dto.value, amplitude_ratio_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of AmplitudeRatio from a DTO unit json representation.

        :param data: The AmplitudeRatio DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "DecibelVolt"}
        :return: A new instance of AmplitudeRatio.
        :rtype: AmplitudeRatio
        """
        return AmplitudeRatio.from_dto(AmplitudeRatioDto.from_json(data))

    def __convert_from_base(self, from_unit: AmplitudeRatioUnits) -> float:
        value = self._value
        
        if from_unit == AmplitudeRatioUnits.DecibelVolt:
            return (value)
        
        if from_unit == AmplitudeRatioUnits.DecibelMicrovolt:
            return (value + 120)
        
        if from_unit == AmplitudeRatioUnits.DecibelMillivolt:
            return (value + 60)
        
        if from_unit == AmplitudeRatioUnits.DecibelUnloaded:
            return (value + 2.218487499)
        
        return None


    def __convert_to_base(self, value: float, to_unit: AmplitudeRatioUnits) -> float:
        
        if to_unit == AmplitudeRatioUnits.DecibelVolt:
            return (value)
        
        if to_unit == AmplitudeRatioUnits.DecibelMicrovolt:
            return (value - 120)
        
        if to_unit == AmplitudeRatioUnits.DecibelMillivolt:
            return (value - 60)
        
        if to_unit == AmplitudeRatioUnits.DecibelUnloaded:
            return (value - 2.218487499)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_decibel_volts(decibel_volts: float):
        """
        Create a new instance of AmplitudeRatio from a value in decibel_volts.

        

        :param meters: The AmplitudeRatio value in decibel_volts.
        :type decibel_volts: float
        :return: A new instance of AmplitudeRatio.
        :rtype: AmplitudeRatio
        """
        return AmplitudeRatio(decibel_volts, AmplitudeRatioUnits.DecibelVolt)

    
    @staticmethod
    def from_decibel_microvolts(decibel_microvolts: float):
        """
        Create a new instance of AmplitudeRatio from a value in decibel_microvolts.

        

        :param meters: The AmplitudeRatio value in decibel_microvolts.
        :type decibel_microvolts: float
        :return: A new instance of AmplitudeRatio.
        :rtype: AmplitudeRatio
        """
        return AmplitudeRatio(decibel_microvolts, AmplitudeRatioUnits.DecibelMicrovolt)

    
    @staticmethod
    def from_decibel_millivolts(decibel_millivolts: float):
        """
        Create a new instance of AmplitudeRatio from a value in decibel_millivolts.

        

        :param meters: The AmplitudeRatio value in decibel_millivolts.
        :type decibel_millivolts: float
        :return: A new instance of AmplitudeRatio.
        :rtype: AmplitudeRatio
        """
        return AmplitudeRatio(decibel_millivolts, AmplitudeRatioUnits.DecibelMillivolt)

    
    @staticmethod
    def from_decibels_unloaded(decibels_unloaded: float):
        """
        Create a new instance of AmplitudeRatio from a value in decibels_unloaded.

        

        :param meters: The AmplitudeRatio value in decibels_unloaded.
        :type decibels_unloaded: float
        :return: A new instance of AmplitudeRatio.
        :rtype: AmplitudeRatio
        """
        return AmplitudeRatio(decibels_unloaded, AmplitudeRatioUnits.DecibelUnloaded)

    
    @property
    def decibel_volts(self) -> float:
        """
        
        """
        if self.__decibel_volts != None:
            return self.__decibel_volts
        self.__decibel_volts = self.__convert_from_base(AmplitudeRatioUnits.DecibelVolt)
        return self.__decibel_volts

    
    @property
    def decibel_microvolts(self) -> float:
        """
        
        """
        if self.__decibel_microvolts != None:
            return self.__decibel_microvolts
        self.__decibel_microvolts = self.__convert_from_base(AmplitudeRatioUnits.DecibelMicrovolt)
        return self.__decibel_microvolts

    
    @property
    def decibel_millivolts(self) -> float:
        """
        
        """
        if self.__decibel_millivolts != None:
            return self.__decibel_millivolts
        self.__decibel_millivolts = self.__convert_from_base(AmplitudeRatioUnits.DecibelMillivolt)
        return self.__decibel_millivolts

    
    @property
    def decibels_unloaded(self) -> float:
        """
        
        """
        if self.__decibels_unloaded != None:
            return self.__decibels_unloaded
        self.__decibels_unloaded = self.__convert_from_base(AmplitudeRatioUnits.DecibelUnloaded)
        return self.__decibels_unloaded

    
    def to_string(self, unit: AmplitudeRatioUnits = AmplitudeRatioUnits.DecibelVolt, fractional_digits: int = None) -> str:
        """
        Format the AmplitudeRatio to a string.
        
        Note: the default format for AmplitudeRatio is DecibelVolt.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the AmplitudeRatio. Default is 'DecibelVolt'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == AmplitudeRatioUnits.DecibelVolt:
            return f"""{super()._truncate_fraction_digits(self.decibel_volts, fractional_digits)} dBV"""
        
        if unit == AmplitudeRatioUnits.DecibelMicrovolt:
            return f"""{super()._truncate_fraction_digits(self.decibel_microvolts, fractional_digits)} dBµV"""
        
        if unit == AmplitudeRatioUnits.DecibelMillivolt:
            return f"""{super()._truncate_fraction_digits(self.decibel_millivolts, fractional_digits)} dBmV"""
        
        if unit == AmplitudeRatioUnits.DecibelUnloaded:
            return f"""{super()._truncate_fraction_digits(self.decibels_unloaded, fractional_digits)} dBu"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: AmplitudeRatioUnits = AmplitudeRatioUnits.DecibelVolt) -> str:
        """
        Get AmplitudeRatio unit abbreviation.
        Note! the default abbreviation for AmplitudeRatio is DecibelVolt.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == AmplitudeRatioUnits.DecibelVolt:
            return """dBV"""
        
        if unit_abbreviation == AmplitudeRatioUnits.DecibelMicrovolt:
            return """dBµV"""
        
        if unit_abbreviation == AmplitudeRatioUnits.DecibelMillivolt:
            return """dBmV"""
        
        if unit_abbreviation == AmplitudeRatioUnits.DecibelUnloaded:
            return """dBu"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class TemperatureDeltaUnits(Enum):
        """
            TemperatureDeltaUnits enumeration
        """
        
        Kelvin = 'Kelvin'
        """
            
        """
        
        DegreeCelsius = 'DegreeCelsius'
        """
            
        """
        
        DegreeDelisle = 'DegreeDelisle'
        """
            
        """
        
        DegreeFahrenheit = 'DegreeFahrenheit'
        """
            
        """
        
        DegreeNewton = 'DegreeNewton'
        """
            
        """
        
        DegreeRankine = 'DegreeRankine'
        """
            
        """
        
        DegreeReaumur = 'DegreeReaumur'
        """
            
        """
        
        DegreeRoemer = 'DegreeRoemer'
        """
            
        """
        
        MillidegreeCelsius = 'MillidegreeCelsius'
        """
            
        """
        

class TemperatureDeltaDto:
    """
    A DTO representation of a TemperatureDelta

    Attributes:
        value (float): The value of the TemperatureDelta.
        unit (TemperatureDeltaUnits): The specific unit that the TemperatureDelta value is representing.
    """

    def __init__(self, value: float, unit: TemperatureDeltaUnits):
        """
        Create a new DTO representation of a TemperatureDelta

        Parameters:
            value (float): The value of the TemperatureDelta.
            unit (TemperatureDeltaUnits): The specific unit that the TemperatureDelta value is representing.
        """
        self.value: float = value
        """
        The value of the TemperatureDelta
        """
        self.unit: TemperatureDeltaUnits = unit
        """
        The specific unit that the TemperatureDelta value is representing
        """

    def to_json(self):
        """
        Get a TemperatureDelta DTO JSON object representing the current unit.

        :return: JSON object represents TemperatureDelta DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "Kelvin"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of TemperatureDelta DTO from a json representation.

        :param data: The TemperatureDelta DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "Kelvin"}
        :return: A new instance of TemperatureDeltaDto.
        :rtype: TemperatureDeltaDto
        """
        return TemperatureDeltaDto(value=data["value"], unit=TemperatureDeltaUnits(data["unit"]))


class TemperatureDelta(AbstractMeasure):
    """
    Difference between two temperatures. The conversions are different than for Temperature.

    Args:
        value (float): The value.
        from_unit (TemperatureDeltaUnits): The TemperatureDelta unit to create from, The default unit is Kelvin
    """
    def __init__(self, value: float, from_unit: TemperatureDeltaUnits = TemperatureDeltaUnits.Kelvin):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__kelvins = None
        
        self.__degrees_celsius = None
        
        self.__degrees_delisle = None
        
        self.__degrees_fahrenheit = None
        
        self.__degrees_newton = None
        
        self.__degrees_rankine = None
        
        self.__degrees_reaumur = None
        
        self.__degrees_roemer = None
        
        self.__millidegrees_celsius = None
        

    def convert(self, unit: TemperatureDeltaUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: TemperatureDeltaUnits = TemperatureDeltaUnits.Kelvin) -> TemperatureDeltaDto:
        """
        Get a new instance of TemperatureDelta DTO representing the current unit.

        :param hold_in_unit: The specific TemperatureDelta unit to store the TemperatureDelta value in the DTO representation.
        :type hold_in_unit: TemperatureDeltaUnits
        :return: A new instance of TemperatureDeltaDto.
        :rtype: TemperatureDeltaDto
        """
        return TemperatureDeltaDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: TemperatureDeltaUnits = TemperatureDeltaUnits.Kelvin):
        """
        Get a TemperatureDelta DTO JSON object representing the current unit.

        :param hold_in_unit: The specific TemperatureDelta unit to store the TemperatureDelta value in the DTO representation.
        :type hold_in_unit: TemperatureDeltaUnits
        :return: JSON object represents TemperatureDelta DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "Kelvin"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(temperature_delta_dto: TemperatureDeltaDto):
        """
        Obtain a new instance of TemperatureDelta from a DTO unit object.

        :param temperature_delta_dto: The TemperatureDelta DTO representation.
        :type temperature_delta_dto: TemperatureDeltaDto
        :return: A new instance of TemperatureDelta.
        :rtype: TemperatureDelta
        """
        return TemperatureDelta(temperature_delta_dto.value, temperature_delta_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of TemperatureDelta from a DTO unit json representation.

        :param data: The TemperatureDelta DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "Kelvin"}
        :return: A new instance of TemperatureDelta.
        :rtype: TemperatureDelta
        """
        return TemperatureDelta.from_dto(TemperatureDeltaDto.from_json(data))

    def __convert_from_base(self, from_unit: TemperatureDeltaUnits) -> float:
        value = self._value
        
        if from_unit == TemperatureDeltaUnits.Kelvin:
            return (value)
        
        if from_unit == TemperatureDeltaUnits.DegreeCelsius:
            return (value)
        
        if from_unit == TemperatureDeltaUnits.DegreeDelisle:
            return (value * -3 / 2)
        
        if from_unit == TemperatureDeltaUnits.DegreeFahrenheit:
            return (value * 9 / 5)
        
        if from_unit == TemperatureDeltaUnits.DegreeNewton:
            return (value * 33 / 100)
        
        if from_unit == TemperatureDeltaUnits.DegreeRankine:
            return (value * 9 / 5)
        
        if from_unit == TemperatureDeltaUnits.DegreeReaumur:
            return (value * 4 / 5)
        
        if from_unit == TemperatureDeltaUnits.DegreeRoemer:
            return (value * 21 / 40)
        
        if from_unit == TemperatureDeltaUnits.MillidegreeCelsius:
            return ((value) / 0.001)
        
        return None


    def __convert_to_base(self, value: float, to_unit: TemperatureDeltaUnits) -> float:
        
        if to_unit == TemperatureDeltaUnits.Kelvin:
            return (value)
        
        if to_unit == TemperatureDeltaUnits.DegreeCelsius:
            return (value)
        
        if to_unit == TemperatureDeltaUnits.DegreeDelisle:
            return (value * -2 / 3)
        
        if to_unit == TemperatureDeltaUnits.DegreeFahrenheit:
            return (value * 5 / 9)
        
        if to_unit == TemperatureDeltaUnits.DegreeNewton:
            return (value * 100 / 33)
        
        if to_unit == TemperatureDeltaUnits.DegreeRankine:
            return (value * 5 / 9)
        
        if to_unit == TemperatureDeltaUnits.DegreeReaumur:
            return (value * 5 / 4)
        
        if to_unit == TemperatureDeltaUnits.DegreeRoemer:
            return (value * 40 / 21)
        
        if to_unit == TemperatureDeltaUnits.MillidegreeCelsius:
            return ((value) * 0.001)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_kelvins(kelvins: float):
        """
        Create a new instance of TemperatureDelta from a value in kelvins.

        

        :param meters: The TemperatureDelta value in kelvins.
        :type kelvins: float
        :return: A new instance of TemperatureDelta.
        :rtype: TemperatureDelta
        """
        return TemperatureDelta(kelvins, TemperatureDeltaUnits.Kelvin)

    
    @staticmethod
    def from_degrees_celsius(degrees_celsius: float):
        """
        Create a new instance of TemperatureDelta from a value in degrees_celsius.

        

        :param meters: The TemperatureDelta value in degrees_celsius.
        :type degrees_celsius: float
        :return: A new instance of TemperatureDelta.
        :rtype: TemperatureDelta
        """
        return TemperatureDelta(degrees_celsius, TemperatureDeltaUnits.DegreeCelsius)

    
    @staticmethod
    def from_degrees_delisle(degrees_delisle: float):
        """
        Create a new instance of TemperatureDelta from a value in degrees_delisle.

        

        :param meters: The TemperatureDelta value in degrees_delisle.
        :type degrees_delisle: float
        :return: A new instance of TemperatureDelta.
        :rtype: TemperatureDelta
        """
        return TemperatureDelta(degrees_delisle, TemperatureDeltaUnits.DegreeDelisle)

    
    @staticmethod
    def from_degrees_fahrenheit(degrees_fahrenheit: float):
        """
        Create a new instance of TemperatureDelta from a value in degrees_fahrenheit.

        

        :param meters: The TemperatureDelta value in degrees_fahrenheit.
        :type degrees_fahrenheit: float
        :return: A new instance of TemperatureDelta.
        :rtype: TemperatureDelta
        """
        return TemperatureDelta(degrees_fahrenheit, TemperatureDeltaUnits.DegreeFahrenheit)

    
    @staticmethod
    def from_degrees_newton(degrees_newton: float):
        """
        Create a new instance of TemperatureDelta from a value in degrees_newton.

        

        :param meters: The TemperatureDelta value in degrees_newton.
        :type degrees_newton: float
        :return: A new instance of TemperatureDelta.
        :rtype: TemperatureDelta
        """
        return TemperatureDelta(degrees_newton, TemperatureDeltaUnits.DegreeNewton)

    
    @staticmethod
    def from_degrees_rankine(degrees_rankine: float):
        """
        Create a new instance of TemperatureDelta from a value in degrees_rankine.

        

        :param meters: The TemperatureDelta value in degrees_rankine.
        :type degrees_rankine: float
        :return: A new instance of TemperatureDelta.
        :rtype: TemperatureDelta
        """
        return TemperatureDelta(degrees_rankine, TemperatureDeltaUnits.DegreeRankine)

    
    @staticmethod
    def from_degrees_reaumur(degrees_reaumur: float):
        """
        Create a new instance of TemperatureDelta from a value in degrees_reaumur.

        

        :param meters: The TemperatureDelta value in degrees_reaumur.
        :type degrees_reaumur: float
        :return: A new instance of TemperatureDelta.
        :rtype: TemperatureDelta
        """
        return TemperatureDelta(degrees_reaumur, TemperatureDeltaUnits.DegreeReaumur)

    
    @staticmethod
    def from_degrees_roemer(degrees_roemer: float):
        """
        Create a new instance of TemperatureDelta from a value in degrees_roemer.

        

        :param meters: The TemperatureDelta value in degrees_roemer.
        :type degrees_roemer: float
        :return: A new instance of TemperatureDelta.
        :rtype: TemperatureDelta
        """
        return TemperatureDelta(degrees_roemer, TemperatureDeltaUnits.DegreeRoemer)

    
    @staticmethod
    def from_millidegrees_celsius(millidegrees_celsius: float):
        """
        Create a new instance of TemperatureDelta from a value in millidegrees_celsius.

        

        :param meters: The TemperatureDelta value in millidegrees_celsius.
        :type millidegrees_celsius: float
        :return: A new instance of TemperatureDelta.
        :rtype: TemperatureDelta
        """
        return TemperatureDelta(millidegrees_celsius, TemperatureDeltaUnits.MillidegreeCelsius)

    
    @property
    def kelvins(self) -> float:
        """
        
        """
        if self.__kelvins != None:
            return self.__kelvins
        self.__kelvins = self.__convert_from_base(TemperatureDeltaUnits.Kelvin)
        return self.__kelvins

    
    @property
    def degrees_celsius(self) -> float:
        """
        
        """
        if self.__degrees_celsius != None:
            return self.__degrees_celsius
        self.__degrees_celsius = self.__convert_from_base(TemperatureDeltaUnits.DegreeCelsius)
        return self.__degrees_celsius

    
    @property
    def degrees_delisle(self) -> float:
        """
        
        """
        if self.__degrees_delisle != None:
            return self.__degrees_delisle
        self.__degrees_delisle = self.__convert_from_base(TemperatureDeltaUnits.DegreeDelisle)
        return self.__degrees_delisle

    
    @property
    def degrees_fahrenheit(self) -> float:
        """
        
        """
        if self.__degrees_fahrenheit != None:
            return self.__degrees_fahrenheit
        self.__degrees_fahrenheit = self.__convert_from_base(TemperatureDeltaUnits.DegreeFahrenheit)
        return self.__degrees_fahrenheit

    
    @property
    def degrees_newton(self) -> float:
        """
        
        """
        if self.__degrees_newton != None:
            return self.__degrees_newton
        self.__degrees_newton = self.__convert_from_base(TemperatureDeltaUnits.DegreeNewton)
        return self.__degrees_newton

    
    @property
    def degrees_rankine(self) -> float:
        """
        
        """
        if self.__degrees_rankine != None:
            return self.__degrees_rankine
        self.__degrees_rankine = self.__convert_from_base(TemperatureDeltaUnits.DegreeRankine)
        return self.__degrees_rankine

    
    @property
    def degrees_reaumur(self) -> float:
        """
        
        """
        if self.__degrees_reaumur != None:
            return self.__degrees_reaumur
        self.__degrees_reaumur = self.__convert_from_base(TemperatureDeltaUnits.DegreeReaumur)
        return self.__degrees_reaumur

    
    @property
    def degrees_roemer(self) -> float:
        """
        
        """
        if self.__degrees_roemer != None:
            return self.__degrees_roemer
        self.__degrees_roemer = self.__convert_from_base(TemperatureDeltaUnits.DegreeRoemer)
        return self.__degrees_roemer

    
    @property
    def millidegrees_celsius(self) -> float:
        """
        
        """
        if self.__millidegrees_celsius != None:
            return self.__millidegrees_celsius
        self.__millidegrees_celsius = self.__convert_from_base(TemperatureDeltaUnits.MillidegreeCelsius)
        return self.__millidegrees_celsius

    
    def to_string(self, unit: TemperatureDeltaUnits = TemperatureDeltaUnits.Kelvin, fractional_digits: int = None) -> str:
        """
        Format the TemperatureDelta to a string.
        
        Note: the default format for TemperatureDelta is Kelvin.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the TemperatureDelta. Default is 'Kelvin'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == TemperatureDeltaUnits.Kelvin:
            return f"""{super()._truncate_fraction_digits(self.kelvins, fractional_digits)} ∆K"""
        
        if unit == TemperatureDeltaUnits.DegreeCelsius:
            return f"""{super()._truncate_fraction_digits(self.degrees_celsius, fractional_digits)} ∆°C"""
        
        if unit == TemperatureDeltaUnits.DegreeDelisle:
            return f"""{super()._truncate_fraction_digits(self.degrees_delisle, fractional_digits)} ∆°De"""
        
        if unit == TemperatureDeltaUnits.DegreeFahrenheit:
            return f"""{super()._truncate_fraction_digits(self.degrees_fahrenheit, fractional_digits)} ∆°F"""
        
        if unit == TemperatureDeltaUnits.DegreeNewton:
            return f"""{super()._truncate_fraction_digits(self.degrees_newton, fractional_digits)} ∆°N"""
        
        if unit == TemperatureDeltaUnits.DegreeRankine:
            return f"""{super()._truncate_fraction_digits(self.degrees_rankine, fractional_digits)} ∆°R"""
        
        if unit == TemperatureDeltaUnits.DegreeReaumur:
            return f"""{super()._truncate_fraction_digits(self.degrees_reaumur, fractional_digits)} ∆°Ré"""
        
        if unit == TemperatureDeltaUnits.DegreeRoemer:
            return f"""{super()._truncate_fraction_digits(self.degrees_roemer, fractional_digits)} ∆°Rø"""
        
        if unit == TemperatureDeltaUnits.MillidegreeCelsius:
            return f"""{super()._truncate_fraction_digits(self.millidegrees_celsius, fractional_digits)} m∆°C"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: TemperatureDeltaUnits = TemperatureDeltaUnits.Kelvin) -> str:
        """
        Get TemperatureDelta unit abbreviation.
        Note! the default abbreviation for TemperatureDelta is Kelvin.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == TemperatureDeltaUnits.Kelvin:
            return """∆K"""
        
        if unit_abbreviation == TemperatureDeltaUnits.DegreeCelsius:
            return """∆°C"""
        
        if unit_abbreviation == TemperatureDeltaUnits.DegreeDelisle:
            return """∆°De"""
        
        if unit_abbreviation == TemperatureDeltaUnits.DegreeFahrenheit:
            return """∆°F"""
        
        if unit_abbreviation == TemperatureDeltaUnits.DegreeNewton:
            return """∆°N"""
        
        if unit_abbreviation == TemperatureDeltaUnits.DegreeRankine:
            return """∆°R"""
        
        if unit_abbreviation == TemperatureDeltaUnits.DegreeReaumur:
            return """∆°Ré"""
        
        if unit_abbreviation == TemperatureDeltaUnits.DegreeRoemer:
            return """∆°Rø"""
        
        if unit_abbreviation == TemperatureDeltaUnits.MillidegreeCelsius:
            return """m∆°C"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class ElectricResistivityUnits(Enum):
        """
            ElectricResistivityUnits enumeration
        """
        
        OhmMeter = 'OhmMeter'
        """
            
        """
        
        OhmCentimeter = 'OhmCentimeter'
        """
            
        """
        
        PicoohmMeter = 'PicoohmMeter'
        """
            
        """
        
        NanoohmMeter = 'NanoohmMeter'
        """
            
        """
        
        MicroohmMeter = 'MicroohmMeter'
        """
            
        """
        
        MilliohmMeter = 'MilliohmMeter'
        """
            
        """
        
        KiloohmMeter = 'KiloohmMeter'
        """
            
        """
        
        MegaohmMeter = 'MegaohmMeter'
        """
            
        """
        
        PicoohmCentimeter = 'PicoohmCentimeter'
        """
            
        """
        
        NanoohmCentimeter = 'NanoohmCentimeter'
        """
            
        """
        
        MicroohmCentimeter = 'MicroohmCentimeter'
        """
            
        """
        
        MilliohmCentimeter = 'MilliohmCentimeter'
        """
            
        """
        
        KiloohmCentimeter = 'KiloohmCentimeter'
        """
            
        """
        
        MegaohmCentimeter = 'MegaohmCentimeter'
        """
            
        """
        

class ElectricResistivityDto:
    """
    A DTO representation of a ElectricResistivity

    Attributes:
        value (float): The value of the ElectricResistivity.
        unit (ElectricResistivityUnits): The specific unit that the ElectricResistivity value is representing.
    """

    def __init__(self, value: float, unit: ElectricResistivityUnits):
        """
        Create a new DTO representation of a ElectricResistivity

        Parameters:
            value (float): The value of the ElectricResistivity.
            unit (ElectricResistivityUnits): The specific unit that the ElectricResistivity value is representing.
        """
        self.value: float = value
        """
        The value of the ElectricResistivity
        """
        self.unit: ElectricResistivityUnits = unit
        """
        The specific unit that the ElectricResistivity value is representing
        """

    def to_json(self):
        """
        Get a ElectricResistivity DTO JSON object representing the current unit.

        :return: JSON object represents ElectricResistivity DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "OhmMeter"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of ElectricResistivity DTO from a json representation.

        :param data: The ElectricResistivity DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "OhmMeter"}
        :return: A new instance of ElectricResistivityDto.
        :rtype: ElectricResistivityDto
        """
        return ElectricResistivityDto(value=data["value"], unit=ElectricResistivityUnits(data["unit"]))


class ElectricResistivity(AbstractMeasure):
    """
    Electrical resistivity (also known as resistivity, specific electrical resistance, or volume resistivity) is a fundamental property that quantifies how strongly a given material opposes the flow of electric current.

    Args:
        value (float): The value.
        from_unit (ElectricResistivityUnits): The ElectricResistivity unit to create from, The default unit is OhmMeter
    """
    def __init__(self, value: float, from_unit: ElectricResistivityUnits = ElectricResistivityUnits.OhmMeter):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__ohm_meters = None
        
        self.__ohms_centimeter = None
        
        self.__picoohm_meters = None
        
        self.__nanoohm_meters = None
        
        self.__microohm_meters = None
        
        self.__milliohm_meters = None
        
        self.__kiloohm_meters = None
        
        self.__megaohm_meters = None
        
        self.__picoohms_centimeter = None
        
        self.__nanoohms_centimeter = None
        
        self.__microohms_centimeter = None
        
        self.__milliohms_centimeter = None
        
        self.__kiloohms_centimeter = None
        
        self.__megaohms_centimeter = None
        

    def convert(self, unit: ElectricResistivityUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: ElectricResistivityUnits = ElectricResistivityUnits.OhmMeter) -> ElectricResistivityDto:
        """
        Get a new instance of ElectricResistivity DTO representing the current unit.

        :param hold_in_unit: The specific ElectricResistivity unit to store the ElectricResistivity value in the DTO representation.
        :type hold_in_unit: ElectricResistivityUnits
        :return: A new instance of ElectricResistivityDto.
        :rtype: ElectricResistivityDto
        """
        return ElectricResistivityDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: ElectricResistivityUnits = ElectricResistivityUnits.OhmMeter):
        """
        Get a ElectricResistivity DTO JSON object representing the current unit.

        :param hold_in_unit: The specific ElectricResistivity unit to store the ElectricResistivity value in the DTO representation.
        :type hold_in_unit: ElectricResistivityUnits
        :return: JSON object represents ElectricResistivity DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "OhmMeter"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(electric_resistivity_dto: ElectricResistivityDto):
        """
        Obtain a new instance of ElectricResistivity from a DTO unit object.

        :param electric_resistivity_dto: The ElectricResistivity DTO representation.
        :type electric_resistivity_dto: ElectricResistivityDto
        :return: A new instance of ElectricResistivity.
        :rtype: ElectricResistivity
        """
        return ElectricResistivity(electric_resistivity_dto.value, electric_resistivity_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of ElectricResistivity from a DTO unit json representation.

        :param data: The ElectricResistivity DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "OhmMeter"}
        :return: A new instance of ElectricResistivity.
        :rtype: ElectricResistivity
        """
        return ElectricResistivity.from_dto(ElectricResistivityDto.from_json(data))

    def __convert_from_base(self, from_unit: ElectricResistivityUnits) -> float:
        value = self._value
        
        if from_unit == ElectricResistivityUnits.OhmMeter:
            return (value)
        
        if from_unit == ElectricResistivityUnits.OhmCentimeter:
            return (value * 100)
        
        if from_unit == ElectricResistivityUnits.PicoohmMeter:
            return ((value) / 1e-12)
        
        if from_unit == ElectricResistivityUnits.NanoohmMeter:
            return ((value) / 1e-09)
        
        if from_unit == ElectricResistivityUnits.MicroohmMeter:
            return ((value) / 1e-06)
        
        if from_unit == ElectricResistivityUnits.MilliohmMeter:
            return ((value) / 0.001)
        
        if from_unit == ElectricResistivityUnits.KiloohmMeter:
            return ((value) / 1000.0)
        
        if from_unit == ElectricResistivityUnits.MegaohmMeter:
            return ((value) / 1000000.0)
        
        if from_unit == ElectricResistivityUnits.PicoohmCentimeter:
            return ((value * 100) / 1e-12)
        
        if from_unit == ElectricResistivityUnits.NanoohmCentimeter:
            return ((value * 100) / 1e-09)
        
        if from_unit == ElectricResistivityUnits.MicroohmCentimeter:
            return ((value * 100) / 1e-06)
        
        if from_unit == ElectricResistivityUnits.MilliohmCentimeter:
            return ((value * 100) / 0.001)
        
        if from_unit == ElectricResistivityUnits.KiloohmCentimeter:
            return ((value * 100) / 1000.0)
        
        if from_unit == ElectricResistivityUnits.MegaohmCentimeter:
            return ((value * 100) / 1000000.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: ElectricResistivityUnits) -> float:
        
        if to_unit == ElectricResistivityUnits.OhmMeter:
            return (value)
        
        if to_unit == ElectricResistivityUnits.OhmCentimeter:
            return (value / 100)
        
        if to_unit == ElectricResistivityUnits.PicoohmMeter:
            return ((value) * 1e-12)
        
        if to_unit == ElectricResistivityUnits.NanoohmMeter:
            return ((value) * 1e-09)
        
        if to_unit == ElectricResistivityUnits.MicroohmMeter:
            return ((value) * 1e-06)
        
        if to_unit == ElectricResistivityUnits.MilliohmMeter:
            return ((value) * 0.001)
        
        if to_unit == ElectricResistivityUnits.KiloohmMeter:
            return ((value) * 1000.0)
        
        if to_unit == ElectricResistivityUnits.MegaohmMeter:
            return ((value) * 1000000.0)
        
        if to_unit == ElectricResistivityUnits.PicoohmCentimeter:
            return ((value / 100) * 1e-12)
        
        if to_unit == ElectricResistivityUnits.NanoohmCentimeter:
            return ((value / 100) * 1e-09)
        
        if to_unit == ElectricResistivityUnits.MicroohmCentimeter:
            return ((value / 100) * 1e-06)
        
        if to_unit == ElectricResistivityUnits.MilliohmCentimeter:
            return ((value / 100) * 0.001)
        
        if to_unit == ElectricResistivityUnits.KiloohmCentimeter:
            return ((value / 100) * 1000.0)
        
        if to_unit == ElectricResistivityUnits.MegaohmCentimeter:
            return ((value / 100) * 1000000.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_ohm_meters(ohm_meters: float):
        """
        Create a new instance of ElectricResistivity from a value in ohm_meters.

        

        :param meters: The ElectricResistivity value in ohm_meters.
        :type ohm_meters: float
        :return: A new instance of ElectricResistivity.
        :rtype: ElectricResistivity
        """
        return ElectricResistivity(ohm_meters, ElectricResistivityUnits.OhmMeter)

    
    @staticmethod
    def from_ohms_centimeter(ohms_centimeter: float):
        """
        Create a new instance of ElectricResistivity from a value in ohms_centimeter.

        

        :param meters: The ElectricResistivity value in ohms_centimeter.
        :type ohms_centimeter: float
        :return: A new instance of ElectricResistivity.
        :rtype: ElectricResistivity
        """
        return ElectricResistivity(ohms_centimeter, ElectricResistivityUnits.OhmCentimeter)

    
    @staticmethod
    def from_picoohm_meters(picoohm_meters: float):
        """
        Create a new instance of ElectricResistivity from a value in picoohm_meters.

        

        :param meters: The ElectricResistivity value in picoohm_meters.
        :type picoohm_meters: float
        :return: A new instance of ElectricResistivity.
        :rtype: ElectricResistivity
        """
        return ElectricResistivity(picoohm_meters, ElectricResistivityUnits.PicoohmMeter)

    
    @staticmethod
    def from_nanoohm_meters(nanoohm_meters: float):
        """
        Create a new instance of ElectricResistivity from a value in nanoohm_meters.

        

        :param meters: The ElectricResistivity value in nanoohm_meters.
        :type nanoohm_meters: float
        :return: A new instance of ElectricResistivity.
        :rtype: ElectricResistivity
        """
        return ElectricResistivity(nanoohm_meters, ElectricResistivityUnits.NanoohmMeter)

    
    @staticmethod
    def from_microohm_meters(microohm_meters: float):
        """
        Create a new instance of ElectricResistivity from a value in microohm_meters.

        

        :param meters: The ElectricResistivity value in microohm_meters.
        :type microohm_meters: float
        :return: A new instance of ElectricResistivity.
        :rtype: ElectricResistivity
        """
        return ElectricResistivity(microohm_meters, ElectricResistivityUnits.MicroohmMeter)

    
    @staticmethod
    def from_milliohm_meters(milliohm_meters: float):
        """
        Create a new instance of ElectricResistivity from a value in milliohm_meters.

        

        :param meters: The ElectricResistivity value in milliohm_meters.
        :type milliohm_meters: float
        :return: A new instance of ElectricResistivity.
        :rtype: ElectricResistivity
        """
        return ElectricResistivity(milliohm_meters, ElectricResistivityUnits.MilliohmMeter)

    
    @staticmethod
    def from_kiloohm_meters(kiloohm_meters: float):
        """
        Create a new instance of ElectricResistivity from a value in kiloohm_meters.

        

        :param meters: The ElectricResistivity value in kiloohm_meters.
        :type kiloohm_meters: float
        :return: A new instance of ElectricResistivity.
        :rtype: ElectricResistivity
        """
        return ElectricResistivity(kiloohm_meters, ElectricResistivityUnits.KiloohmMeter)

    
    @staticmethod
    def from_megaohm_meters(megaohm_meters: float):
        """
        Create a new instance of ElectricResistivity from a value in megaohm_meters.

        

        :param meters: The ElectricResistivity value in megaohm_meters.
        :type megaohm_meters: float
        :return: A new instance of ElectricResistivity.
        :rtype: ElectricResistivity
        """
        return ElectricResistivity(megaohm_meters, ElectricResistivityUnits.MegaohmMeter)

    
    @staticmethod
    def from_picoohms_centimeter(picoohms_centimeter: float):
        """
        Create a new instance of ElectricResistivity from a value in picoohms_centimeter.

        

        :param meters: The ElectricResistivity value in picoohms_centimeter.
        :type picoohms_centimeter: float
        :return: A new instance of ElectricResistivity.
        :rtype: ElectricResistivity
        """
        return ElectricResistivity(picoohms_centimeter, ElectricResistivityUnits.PicoohmCentimeter)

    
    @staticmethod
    def from_nanoohms_centimeter(nanoohms_centimeter: float):
        """
        Create a new instance of ElectricResistivity from a value in nanoohms_centimeter.

        

        :param meters: The ElectricResistivity value in nanoohms_centimeter.
        :type nanoohms_centimeter: float
        :return: A new instance of ElectricResistivity.
        :rtype: ElectricResistivity
        """
        return ElectricResistivity(nanoohms_centimeter, ElectricResistivityUnits.NanoohmCentimeter)

    
    @staticmethod
    def from_microohms_centimeter(microohms_centimeter: float):
        """
        Create a new instance of ElectricResistivity from a value in microohms_centimeter.

        

        :param meters: The ElectricResistivity value in microohms_centimeter.
        :type microohms_centimeter: float
        :return: A new instance of ElectricResistivity.
        :rtype: ElectricResistivity
        """
        return ElectricResistivity(microohms_centimeter, ElectricResistivityUnits.MicroohmCentimeter)

    
    @staticmethod
    def from_milliohms_centimeter(milliohms_centimeter: float):
        """
        Create a new instance of ElectricResistivity from a value in milliohms_centimeter.

        

        :param meters: The ElectricResistivity value in milliohms_centimeter.
        :type milliohms_centimeter: float
        :return: A new instance of ElectricResistivity.
        :rtype: ElectricResistivity
        """
        return ElectricResistivity(milliohms_centimeter, ElectricResistivityUnits.MilliohmCentimeter)

    
    @staticmethod
    def from_kiloohms_centimeter(kiloohms_centimeter: float):
        """
        Create a new instance of ElectricResistivity from a value in kiloohms_centimeter.

        

        :param meters: The ElectricResistivity value in kiloohms_centimeter.
        :type kiloohms_centimeter: float
        :return: A new instance of ElectricResistivity.
        :rtype: ElectricResistivity
        """
        return ElectricResistivity(kiloohms_centimeter, ElectricResistivityUnits.KiloohmCentimeter)

    
    @staticmethod
    def from_megaohms_centimeter(megaohms_centimeter: float):
        """
        Create a new instance of ElectricResistivity from a value in megaohms_centimeter.

        

        :param meters: The ElectricResistivity value in megaohms_centimeter.
        :type megaohms_centimeter: float
        :return: A new instance of ElectricResistivity.
        :rtype: ElectricResistivity
        """
        return ElectricResistivity(megaohms_centimeter, ElectricResistivityUnits.MegaohmCentimeter)

    
    @property
    def ohm_meters(self) -> float:
        """
        
        """
        if self.__ohm_meters != None:
            return self.__ohm_meters
        self.__ohm_meters = self.__convert_from_base(ElectricResistivityUnits.OhmMeter)
        return self.__ohm_meters

    
    @property
    def ohms_centimeter(self) -> float:
        """
        
        """
        if self.__ohms_centimeter != None:
            return self.__ohms_centimeter
        self.__ohms_centimeter = self.__convert_from_base(ElectricResistivityUnits.OhmCentimeter)
        return self.__ohms_centimeter

    
    @property
    def picoohm_meters(self) -> float:
        """
        
        """
        if self.__picoohm_meters != None:
            return self.__picoohm_meters
        self.__picoohm_meters = self.__convert_from_base(ElectricResistivityUnits.PicoohmMeter)
        return self.__picoohm_meters

    
    @property
    def nanoohm_meters(self) -> float:
        """
        
        """
        if self.__nanoohm_meters != None:
            return self.__nanoohm_meters
        self.__nanoohm_meters = self.__convert_from_base(ElectricResistivityUnits.NanoohmMeter)
        return self.__nanoohm_meters

    
    @property
    def microohm_meters(self) -> float:
        """
        
        """
        if self.__microohm_meters != None:
            return self.__microohm_meters
        self.__microohm_meters = self.__convert_from_base(ElectricResistivityUnits.MicroohmMeter)
        return self.__microohm_meters

    
    @property
    def milliohm_meters(self) -> float:
        """
        
        """
        if self.__milliohm_meters != None:
            return self.__milliohm_meters
        self.__milliohm_meters = self.__convert_from_base(ElectricResistivityUnits.MilliohmMeter)
        return self.__milliohm_meters

    
    @property
    def kiloohm_meters(self) -> float:
        """
        
        """
        if self.__kiloohm_meters != None:
            return self.__kiloohm_meters
        self.__kiloohm_meters = self.__convert_from_base(ElectricResistivityUnits.KiloohmMeter)
        return self.__kiloohm_meters

    
    @property
    def megaohm_meters(self) -> float:
        """
        
        """
        if self.__megaohm_meters != None:
            return self.__megaohm_meters
        self.__megaohm_meters = self.__convert_from_base(ElectricResistivityUnits.MegaohmMeter)
        return self.__megaohm_meters

    
    @property
    def picoohms_centimeter(self) -> float:
        """
        
        """
        if self.__picoohms_centimeter != None:
            return self.__picoohms_centimeter
        self.__picoohms_centimeter = self.__convert_from_base(ElectricResistivityUnits.PicoohmCentimeter)
        return self.__picoohms_centimeter

    
    @property
    def nanoohms_centimeter(self) -> float:
        """
        
        """
        if self.__nanoohms_centimeter != None:
            return self.__nanoohms_centimeter
        self.__nanoohms_centimeter = self.__convert_from_base(ElectricResistivityUnits.NanoohmCentimeter)
        return self.__nanoohms_centimeter

    
    @property
    def microohms_centimeter(self) -> float:
        """
        
        """
        if self.__microohms_centimeter != None:
            return self.__microohms_centimeter
        self.__microohms_centimeter = self.__convert_from_base(ElectricResistivityUnits.MicroohmCentimeter)
        return self.__microohms_centimeter

    
    @property
    def milliohms_centimeter(self) -> float:
        """
        
        """
        if self.__milliohms_centimeter != None:
            return self.__milliohms_centimeter
        self.__milliohms_centimeter = self.__convert_from_base(ElectricResistivityUnits.MilliohmCentimeter)
        return self.__milliohms_centimeter

    
    @property
    def kiloohms_centimeter(self) -> float:
        """
        
        """
        if self.__kiloohms_centimeter != None:
            return self.__kiloohms_centimeter
        self.__kiloohms_centimeter = self.__convert_from_base(ElectricResistivityUnits.KiloohmCentimeter)
        return self.__kiloohms_centimeter

    
    @property
    def megaohms_centimeter(self) -> float:
        """
        
        """
        if self.__megaohms_centimeter != None:
            return self.__megaohms_centimeter
        self.__megaohms_centimeter = self.__convert_from_base(ElectricResistivityUnits.MegaohmCentimeter)
        return self.__megaohms_centimeter

    
    def to_string(self, unit: ElectricResistivityUnits = ElectricResistivityUnits.OhmMeter, fractional_digits: int = None) -> str:
        """
        Format the ElectricResistivity to a string.
        
        Note: the default format for ElectricResistivity is OhmMeter.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the ElectricResistivity. Default is 'OhmMeter'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == ElectricResistivityUnits.OhmMeter:
            return f"""{super()._truncate_fraction_digits(self.ohm_meters, fractional_digits)} Ω·m"""
        
        if unit == ElectricResistivityUnits.OhmCentimeter:
            return f"""{super()._truncate_fraction_digits(self.ohms_centimeter, fractional_digits)} Ω·cm"""
        
        if unit == ElectricResistivityUnits.PicoohmMeter:
            return f"""{super()._truncate_fraction_digits(self.picoohm_meters, fractional_digits)} pΩ·m"""
        
        if unit == ElectricResistivityUnits.NanoohmMeter:
            return f"""{super()._truncate_fraction_digits(self.nanoohm_meters, fractional_digits)} nΩ·m"""
        
        if unit == ElectricResistivityUnits.MicroohmMeter:
            return f"""{super()._truncate_fraction_digits(self.microohm_meters, fractional_digits)} μΩ·m"""
        
        if unit == ElectricResistivityUnits.MilliohmMeter:
            return f"""{super()._truncate_fraction_digits(self.milliohm_meters, fractional_digits)} mΩ·m"""
        
        if unit == ElectricResistivityUnits.KiloohmMeter:
            return f"""{super()._truncate_fraction_digits(self.kiloohm_meters, fractional_digits)} kΩ·m"""
        
        if unit == ElectricResistivityUnits.MegaohmMeter:
            return f"""{super()._truncate_fraction_digits(self.megaohm_meters, fractional_digits)} MΩ·m"""
        
        if unit == ElectricResistivityUnits.PicoohmCentimeter:
            return f"""{super()._truncate_fraction_digits(self.picoohms_centimeter, fractional_digits)} pΩ·cm"""
        
        if unit == ElectricResistivityUnits.NanoohmCentimeter:
            return f"""{super()._truncate_fraction_digits(self.nanoohms_centimeter, fractional_digits)} nΩ·cm"""
        
        if unit == ElectricResistivityUnits.MicroohmCentimeter:
            return f"""{super()._truncate_fraction_digits(self.microohms_centimeter, fractional_digits)} μΩ·cm"""
        
        if unit == ElectricResistivityUnits.MilliohmCentimeter:
            return f"""{super()._truncate_fraction_digits(self.milliohms_centimeter, fractional_digits)} mΩ·cm"""
        
        if unit == ElectricResistivityUnits.KiloohmCentimeter:
            return f"""{super()._truncate_fraction_digits(self.kiloohms_centimeter, fractional_digits)} kΩ·cm"""
        
        if unit == ElectricResistivityUnits.MegaohmCentimeter:
            return f"""{super()._truncate_fraction_digits(self.megaohms_centimeter, fractional_digits)} MΩ·cm"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: ElectricResistivityUnits = ElectricResistivityUnits.OhmMeter) -> str:
        """
        Get ElectricResistivity unit abbreviation.
        Note! the default abbreviation for ElectricResistivity is OhmMeter.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == ElectricResistivityUnits.OhmMeter:
            return """Ω·m"""
        
        if unit_abbreviation == ElectricResistivityUnits.OhmCentimeter:
            return """Ω·cm"""
        
        if unit_abbreviation == ElectricResistivityUnits.PicoohmMeter:
            return """pΩ·m"""
        
        if unit_abbreviation == ElectricResistivityUnits.NanoohmMeter:
            return """nΩ·m"""
        
        if unit_abbreviation == ElectricResistivityUnits.MicroohmMeter:
            return """μΩ·m"""
        
        if unit_abbreviation == ElectricResistivityUnits.MilliohmMeter:
            return """mΩ·m"""
        
        if unit_abbreviation == ElectricResistivityUnits.KiloohmMeter:
            return """kΩ·m"""
        
        if unit_abbreviation == ElectricResistivityUnits.MegaohmMeter:
            return """MΩ·m"""
        
        if unit_abbreviation == ElectricResistivityUnits.PicoohmCentimeter:
            return """pΩ·cm"""
        
        if unit_abbreviation == ElectricResistivityUnits.NanoohmCentimeter:
            return """nΩ·cm"""
        
        if unit_abbreviation == ElectricResistivityUnits.MicroohmCentimeter:
            return """μΩ·cm"""
        
        if unit_abbreviation == ElectricResistivityUnits.MilliohmCentimeter:
            return """mΩ·cm"""
        
        if unit_abbreviation == ElectricResistivityUnits.KiloohmCentimeter:
            return """kΩ·cm"""
        
        if unit_abbreviation == ElectricResistivityUnits.MegaohmCentimeter:
            return """MΩ·cm"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class RotationalSpeedUnits(Enum):
        """
            RotationalSpeedUnits enumeration
        """
        
        RadianPerSecond = 'RadianPerSecond'
        """
            
        """
        
        DegreePerSecond = 'DegreePerSecond'
        """
            
        """
        
        DegreePerMinute = 'DegreePerMinute'
        """
            
        """
        
        RevolutionPerSecond = 'RevolutionPerSecond'
        """
            
        """
        
        RevolutionPerMinute = 'RevolutionPerMinute'
        """
            
        """
        
        NanoradianPerSecond = 'NanoradianPerSecond'
        """
            
        """
        
        MicroradianPerSecond = 'MicroradianPerSecond'
        """
            
        """
        
        MilliradianPerSecond = 'MilliradianPerSecond'
        """
            
        """
        
        CentiradianPerSecond = 'CentiradianPerSecond'
        """
            
        """
        
        DeciradianPerSecond = 'DeciradianPerSecond'
        """
            
        """
        
        NanodegreePerSecond = 'NanodegreePerSecond'
        """
            
        """
        
        MicrodegreePerSecond = 'MicrodegreePerSecond'
        """
            
        """
        
        MillidegreePerSecond = 'MillidegreePerSecond'
        """
            
        """
        

class RotationalSpeedDto:
    """
    A DTO representation of a RotationalSpeed

    Attributes:
        value (float): The value of the RotationalSpeed.
        unit (RotationalSpeedUnits): The specific unit that the RotationalSpeed value is representing.
    """

    def __init__(self, value: float, unit: RotationalSpeedUnits):
        """
        Create a new DTO representation of a RotationalSpeed

        Parameters:
            value (float): The value of the RotationalSpeed.
            unit (RotationalSpeedUnits): The specific unit that the RotationalSpeed value is representing.
        """
        self.value: float = value
        """
        The value of the RotationalSpeed
        """
        self.unit: RotationalSpeedUnits = unit
        """
        The specific unit that the RotationalSpeed value is representing
        """

    def to_json(self):
        """
        Get a RotationalSpeed DTO JSON object representing the current unit.

        :return: JSON object represents RotationalSpeed DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "RadianPerSecond"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of RotationalSpeed DTO from a json representation.

        :param data: The RotationalSpeed DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "RadianPerSecond"}
        :return: A new instance of RotationalSpeedDto.
        :rtype: RotationalSpeedDto
        """
        return RotationalSpeedDto(value=data["value"], unit=RotationalSpeedUnits(data["unit"]))


class RotationalSpeed(AbstractMeasure):
    """
    Rotational speed (sometimes called speed of revolution) is the number of complete rotations, revolutions, cycles, or turns per time unit. Rotational speed is a cyclic frequency, measured in radians per second or in hertz in the SI System by scientists, or in revolutions per minute (rpm or min-1) or revolutions per second in everyday life. The symbol for rotational speed is ω (the Greek lowercase letter "omega").

    Args:
        value (float): The value.
        from_unit (RotationalSpeedUnits): The RotationalSpeed unit to create from, The default unit is RadianPerSecond
    """
    def __init__(self, value: float, from_unit: RotationalSpeedUnits = RotationalSpeedUnits.RadianPerSecond):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__radians_per_second = None
        
        self.__degrees_per_second = None
        
        self.__degrees_per_minute = None
        
        self.__revolutions_per_second = None
        
        self.__revolutions_per_minute = None
        
        self.__nanoradians_per_second = None
        
        self.__microradians_per_second = None
        
        self.__milliradians_per_second = None
        
        self.__centiradians_per_second = None
        
        self.__deciradians_per_second = None
        
        self.__nanodegrees_per_second = None
        
        self.__microdegrees_per_second = None
        
        self.__millidegrees_per_second = None
        

    def convert(self, unit: RotationalSpeedUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: RotationalSpeedUnits = RotationalSpeedUnits.RadianPerSecond) -> RotationalSpeedDto:
        """
        Get a new instance of RotationalSpeed DTO representing the current unit.

        :param hold_in_unit: The specific RotationalSpeed unit to store the RotationalSpeed value in the DTO representation.
        :type hold_in_unit: RotationalSpeedUnits
        :return: A new instance of RotationalSpeedDto.
        :rtype: RotationalSpeedDto
        """
        return RotationalSpeedDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: RotationalSpeedUnits = RotationalSpeedUnits.RadianPerSecond):
        """
        Get a RotationalSpeed DTO JSON object representing the current unit.

        :param hold_in_unit: The specific RotationalSpeed unit to store the RotationalSpeed value in the DTO representation.
        :type hold_in_unit: RotationalSpeedUnits
        :return: JSON object represents RotationalSpeed DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "RadianPerSecond"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(rotational_speed_dto: RotationalSpeedDto):
        """
        Obtain a new instance of RotationalSpeed from a DTO unit object.

        :param rotational_speed_dto: The RotationalSpeed DTO representation.
        :type rotational_speed_dto: RotationalSpeedDto
        :return: A new instance of RotationalSpeed.
        :rtype: RotationalSpeed
        """
        return RotationalSpeed(rotational_speed_dto.value, rotational_speed_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of RotationalSpeed from a DTO unit json representation.

        :param data: The RotationalSpeed DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "RadianPerSecond"}
        :return: A new instance of RotationalSpeed.
        :rtype: RotationalSpeed
        """
        return RotationalSpeed.from_dto(RotationalSpeedDto.from_json(data))

    def __convert_from_base(self, from_unit: RotationalSpeedUnits) -> float:
        value = self._value
        
        if from_unit == RotationalSpeedUnits.RadianPerSecond:
            return (value)
        
        if from_unit == RotationalSpeedUnits.DegreePerSecond:
            return ((180 / math.pi) * value)
        
        if from_unit == RotationalSpeedUnits.DegreePerMinute:
            return ((180 * 60 / math.pi) * value)
        
        if from_unit == RotationalSpeedUnits.RevolutionPerSecond:
            return (value / 6.2831853072)
        
        if from_unit == RotationalSpeedUnits.RevolutionPerMinute:
            return ((value / 6.2831853072) * 60)
        
        if from_unit == RotationalSpeedUnits.NanoradianPerSecond:
            return ((value) / 1e-09)
        
        if from_unit == RotationalSpeedUnits.MicroradianPerSecond:
            return ((value) / 1e-06)
        
        if from_unit == RotationalSpeedUnits.MilliradianPerSecond:
            return ((value) / 0.001)
        
        if from_unit == RotationalSpeedUnits.CentiradianPerSecond:
            return ((value) / 0.01)
        
        if from_unit == RotationalSpeedUnits.DeciradianPerSecond:
            return ((value) / 0.1)
        
        if from_unit == RotationalSpeedUnits.NanodegreePerSecond:
            return (((180 / math.pi) * value) / 1e-09)
        
        if from_unit == RotationalSpeedUnits.MicrodegreePerSecond:
            return (((180 / math.pi) * value) / 1e-06)
        
        if from_unit == RotationalSpeedUnits.MillidegreePerSecond:
            return (((180 / math.pi) * value) / 0.001)
        
        return None


    def __convert_to_base(self, value: float, to_unit: RotationalSpeedUnits) -> float:
        
        if to_unit == RotationalSpeedUnits.RadianPerSecond:
            return (value)
        
        if to_unit == RotationalSpeedUnits.DegreePerSecond:
            return ((math.pi / 180) * value)
        
        if to_unit == RotationalSpeedUnits.DegreePerMinute:
            return ((math.pi / (180 * 60)) * value)
        
        if to_unit == RotationalSpeedUnits.RevolutionPerSecond:
            return (value * 6.2831853072)
        
        if to_unit == RotationalSpeedUnits.RevolutionPerMinute:
            return ((value * 6.2831853072) / 60)
        
        if to_unit == RotationalSpeedUnits.NanoradianPerSecond:
            return ((value) * 1e-09)
        
        if to_unit == RotationalSpeedUnits.MicroradianPerSecond:
            return ((value) * 1e-06)
        
        if to_unit == RotationalSpeedUnits.MilliradianPerSecond:
            return ((value) * 0.001)
        
        if to_unit == RotationalSpeedUnits.CentiradianPerSecond:
            return ((value) * 0.01)
        
        if to_unit == RotationalSpeedUnits.DeciradianPerSecond:
            return ((value) * 0.1)
        
        if to_unit == RotationalSpeedUnits.NanodegreePerSecond:
            return (((math.pi / 180) * value) * 1e-09)
        
        if to_unit == RotationalSpeedUnits.MicrodegreePerSecond:
            return (((math.pi / 180) * value) * 1e-06)
        
        if to_unit == RotationalSpeedUnits.MillidegreePerSecond:
            return (((math.pi / 180) * value) * 0.001)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_radians_per_second(radians_per_second: float):
        """
        Create a new instance of RotationalSpeed from a value in radians_per_second.

        

        :param meters: The RotationalSpeed value in radians_per_second.
        :type radians_per_second: float
        :return: A new instance of RotationalSpeed.
        :rtype: RotationalSpeed
        """
        return RotationalSpeed(radians_per_second, RotationalSpeedUnits.RadianPerSecond)

    
    @staticmethod
    def from_degrees_per_second(degrees_per_second: float):
        """
        Create a new instance of RotationalSpeed from a value in degrees_per_second.

        

        :param meters: The RotationalSpeed value in degrees_per_second.
        :type degrees_per_second: float
        :return: A new instance of RotationalSpeed.
        :rtype: RotationalSpeed
        """
        return RotationalSpeed(degrees_per_second, RotationalSpeedUnits.DegreePerSecond)

    
    @staticmethod
    def from_degrees_per_minute(degrees_per_minute: float):
        """
        Create a new instance of RotationalSpeed from a value in degrees_per_minute.

        

        :param meters: The RotationalSpeed value in degrees_per_minute.
        :type degrees_per_minute: float
        :return: A new instance of RotationalSpeed.
        :rtype: RotationalSpeed
        """
        return RotationalSpeed(degrees_per_minute, RotationalSpeedUnits.DegreePerMinute)

    
    @staticmethod
    def from_revolutions_per_second(revolutions_per_second: float):
        """
        Create a new instance of RotationalSpeed from a value in revolutions_per_second.

        

        :param meters: The RotationalSpeed value in revolutions_per_second.
        :type revolutions_per_second: float
        :return: A new instance of RotationalSpeed.
        :rtype: RotationalSpeed
        """
        return RotationalSpeed(revolutions_per_second, RotationalSpeedUnits.RevolutionPerSecond)

    
    @staticmethod
    def from_revolutions_per_minute(revolutions_per_minute: float):
        """
        Create a new instance of RotationalSpeed from a value in revolutions_per_minute.

        

        :param meters: The RotationalSpeed value in revolutions_per_minute.
        :type revolutions_per_minute: float
        :return: A new instance of RotationalSpeed.
        :rtype: RotationalSpeed
        """
        return RotationalSpeed(revolutions_per_minute, RotationalSpeedUnits.RevolutionPerMinute)

    
    @staticmethod
    def from_nanoradians_per_second(nanoradians_per_second: float):
        """
        Create a new instance of RotationalSpeed from a value in nanoradians_per_second.

        

        :param meters: The RotationalSpeed value in nanoradians_per_second.
        :type nanoradians_per_second: float
        :return: A new instance of RotationalSpeed.
        :rtype: RotationalSpeed
        """
        return RotationalSpeed(nanoradians_per_second, RotationalSpeedUnits.NanoradianPerSecond)

    
    @staticmethod
    def from_microradians_per_second(microradians_per_second: float):
        """
        Create a new instance of RotationalSpeed from a value in microradians_per_second.

        

        :param meters: The RotationalSpeed value in microradians_per_second.
        :type microradians_per_second: float
        :return: A new instance of RotationalSpeed.
        :rtype: RotationalSpeed
        """
        return RotationalSpeed(microradians_per_second, RotationalSpeedUnits.MicroradianPerSecond)

    
    @staticmethod
    def from_milliradians_per_second(milliradians_per_second: float):
        """
        Create a new instance of RotationalSpeed from a value in milliradians_per_second.

        

        :param meters: The RotationalSpeed value in milliradians_per_second.
        :type milliradians_per_second: float
        :return: A new instance of RotationalSpeed.
        :rtype: RotationalSpeed
        """
        return RotationalSpeed(milliradians_per_second, RotationalSpeedUnits.MilliradianPerSecond)

    
    @staticmethod
    def from_centiradians_per_second(centiradians_per_second: float):
        """
        Create a new instance of RotationalSpeed from a value in centiradians_per_second.

        

        :param meters: The RotationalSpeed value in centiradians_per_second.
        :type centiradians_per_second: float
        :return: A new instance of RotationalSpeed.
        :rtype: RotationalSpeed
        """
        return RotationalSpeed(centiradians_per_second, RotationalSpeedUnits.CentiradianPerSecond)

    
    @staticmethod
    def from_deciradians_per_second(deciradians_per_second: float):
        """
        Create a new instance of RotationalSpeed from a value in deciradians_per_second.

        

        :param meters: The RotationalSpeed value in deciradians_per_second.
        :type deciradians_per_second: float
        :return: A new instance of RotationalSpeed.
        :rtype: RotationalSpeed
        """
        return RotationalSpeed(deciradians_per_second, RotationalSpeedUnits.DeciradianPerSecond)

    
    @staticmethod
    def from_nanodegrees_per_second(nanodegrees_per_second: float):
        """
        Create a new instance of RotationalSpeed from a value in nanodegrees_per_second.

        

        :param meters: The RotationalSpeed value in nanodegrees_per_second.
        :type nanodegrees_per_second: float
        :return: A new instance of RotationalSpeed.
        :rtype: RotationalSpeed
        """
        return RotationalSpeed(nanodegrees_per_second, RotationalSpeedUnits.NanodegreePerSecond)

    
    @staticmethod
    def from_microdegrees_per_second(microdegrees_per_second: float):
        """
        Create a new instance of RotationalSpeed from a value in microdegrees_per_second.

        

        :param meters: The RotationalSpeed value in microdegrees_per_second.
        :type microdegrees_per_second: float
        :return: A new instance of RotationalSpeed.
        :rtype: RotationalSpeed
        """
        return RotationalSpeed(microdegrees_per_second, RotationalSpeedUnits.MicrodegreePerSecond)

    
    @staticmethod
    def from_millidegrees_per_second(millidegrees_per_second: float):
        """
        Create a new instance of RotationalSpeed from a value in millidegrees_per_second.

        

        :param meters: The RotationalSpeed value in millidegrees_per_second.
        :type millidegrees_per_second: float
        :return: A new instance of RotationalSpeed.
        :rtype: RotationalSpeed
        """
        return RotationalSpeed(millidegrees_per_second, RotationalSpeedUnits.MillidegreePerSecond)

    
    @property
    def radians_per_second(self) -> float:
        """
        
        """
        if self.__radians_per_second != None:
            return self.__radians_per_second
        self.__radians_per_second = self.__convert_from_base(RotationalSpeedUnits.RadianPerSecond)
        return self.__radians_per_second

    
    @property
    def degrees_per_second(self) -> float:
        """
        
        """
        if self.__degrees_per_second != None:
            return self.__degrees_per_second
        self.__degrees_per_second = self.__convert_from_base(RotationalSpeedUnits.DegreePerSecond)
        return self.__degrees_per_second

    
    @property
    def degrees_per_minute(self) -> float:
        """
        
        """
        if self.__degrees_per_minute != None:
            return self.__degrees_per_minute
        self.__degrees_per_minute = self.__convert_from_base(RotationalSpeedUnits.DegreePerMinute)
        return self.__degrees_per_minute

    
    @property
    def revolutions_per_second(self) -> float:
        """
        
        """
        if self.__revolutions_per_second != None:
            return self.__revolutions_per_second
        self.__revolutions_per_second = self.__convert_from_base(RotationalSpeedUnits.RevolutionPerSecond)
        return self.__revolutions_per_second

    
    @property
    def revolutions_per_minute(self) -> float:
        """
        
        """
        if self.__revolutions_per_minute != None:
            return self.__revolutions_per_minute
        self.__revolutions_per_minute = self.__convert_from_base(RotationalSpeedUnits.RevolutionPerMinute)
        return self.__revolutions_per_minute

    
    @property
    def nanoradians_per_second(self) -> float:
        """
        
        """
        if self.__nanoradians_per_second != None:
            return self.__nanoradians_per_second
        self.__nanoradians_per_second = self.__convert_from_base(RotationalSpeedUnits.NanoradianPerSecond)
        return self.__nanoradians_per_second

    
    @property
    def microradians_per_second(self) -> float:
        """
        
        """
        if self.__microradians_per_second != None:
            return self.__microradians_per_second
        self.__microradians_per_second = self.__convert_from_base(RotationalSpeedUnits.MicroradianPerSecond)
        return self.__microradians_per_second

    
    @property
    def milliradians_per_second(self) -> float:
        """
        
        """
        if self.__milliradians_per_second != None:
            return self.__milliradians_per_second
        self.__milliradians_per_second = self.__convert_from_base(RotationalSpeedUnits.MilliradianPerSecond)
        return self.__milliradians_per_second

    
    @property
    def centiradians_per_second(self) -> float:
        """
        
        """
        if self.__centiradians_per_second != None:
            return self.__centiradians_per_second
        self.__centiradians_per_second = self.__convert_from_base(RotationalSpeedUnits.CentiradianPerSecond)
        return self.__centiradians_per_second

    
    @property
    def deciradians_per_second(self) -> float:
        """
        
        """
        if self.__deciradians_per_second != None:
            return self.__deciradians_per_second
        self.__deciradians_per_second = self.__convert_from_base(RotationalSpeedUnits.DeciradianPerSecond)
        return self.__deciradians_per_second

    
    @property
    def nanodegrees_per_second(self) -> float:
        """
        
        """
        if self.__nanodegrees_per_second != None:
            return self.__nanodegrees_per_second
        self.__nanodegrees_per_second = self.__convert_from_base(RotationalSpeedUnits.NanodegreePerSecond)
        return self.__nanodegrees_per_second

    
    @property
    def microdegrees_per_second(self) -> float:
        """
        
        """
        if self.__microdegrees_per_second != None:
            return self.__microdegrees_per_second
        self.__microdegrees_per_second = self.__convert_from_base(RotationalSpeedUnits.MicrodegreePerSecond)
        return self.__microdegrees_per_second

    
    @property
    def millidegrees_per_second(self) -> float:
        """
        
        """
        if self.__millidegrees_per_second != None:
            return self.__millidegrees_per_second
        self.__millidegrees_per_second = self.__convert_from_base(RotationalSpeedUnits.MillidegreePerSecond)
        return self.__millidegrees_per_second

    
    def to_string(self, unit: RotationalSpeedUnits = RotationalSpeedUnits.RadianPerSecond, fractional_digits: int = None) -> str:
        """
        Format the RotationalSpeed to a string.
        
        Note: the default format for RotationalSpeed is RadianPerSecond.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the RotationalSpeed. Default is 'RadianPerSecond'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == RotationalSpeedUnits.RadianPerSecond:
            return f"""{super()._truncate_fraction_digits(self.radians_per_second, fractional_digits)} rad/s"""
        
        if unit == RotationalSpeedUnits.DegreePerSecond:
            return f"""{super()._truncate_fraction_digits(self.degrees_per_second, fractional_digits)} °/s"""
        
        if unit == RotationalSpeedUnits.DegreePerMinute:
            return f"""{super()._truncate_fraction_digits(self.degrees_per_minute, fractional_digits)} °/min"""
        
        if unit == RotationalSpeedUnits.RevolutionPerSecond:
            return f"""{super()._truncate_fraction_digits(self.revolutions_per_second, fractional_digits)} r/s"""
        
        if unit == RotationalSpeedUnits.RevolutionPerMinute:
            return f"""{super()._truncate_fraction_digits(self.revolutions_per_minute, fractional_digits)} rpm"""
        
        if unit == RotationalSpeedUnits.NanoradianPerSecond:
            return f"""{super()._truncate_fraction_digits(self.nanoradians_per_second, fractional_digits)} nrad/s"""
        
        if unit == RotationalSpeedUnits.MicroradianPerSecond:
            return f"""{super()._truncate_fraction_digits(self.microradians_per_second, fractional_digits)} μrad/s"""
        
        if unit == RotationalSpeedUnits.MilliradianPerSecond:
            return f"""{super()._truncate_fraction_digits(self.milliradians_per_second, fractional_digits)} mrad/s"""
        
        if unit == RotationalSpeedUnits.CentiradianPerSecond:
            return f"""{super()._truncate_fraction_digits(self.centiradians_per_second, fractional_digits)} crad/s"""
        
        if unit == RotationalSpeedUnits.DeciradianPerSecond:
            return f"""{super()._truncate_fraction_digits(self.deciradians_per_second, fractional_digits)} drad/s"""
        
        if unit == RotationalSpeedUnits.NanodegreePerSecond:
            return f"""{super()._truncate_fraction_digits(self.nanodegrees_per_second, fractional_digits)} n°/s"""
        
        if unit == RotationalSpeedUnits.MicrodegreePerSecond:
            return f"""{super()._truncate_fraction_digits(self.microdegrees_per_second, fractional_digits)} μ°/s"""
        
        if unit == RotationalSpeedUnits.MillidegreePerSecond:
            return f"""{super()._truncate_fraction_digits(self.millidegrees_per_second, fractional_digits)} m°/s"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: RotationalSpeedUnits = RotationalSpeedUnits.RadianPerSecond) -> str:
        """
        Get RotationalSpeed unit abbreviation.
        Note! the default abbreviation for RotationalSpeed is RadianPerSecond.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == RotationalSpeedUnits.RadianPerSecond:
            return """rad/s"""
        
        if unit_abbreviation == RotationalSpeedUnits.DegreePerSecond:
            return """°/s"""
        
        if unit_abbreviation == RotationalSpeedUnits.DegreePerMinute:
            return """°/min"""
        
        if unit_abbreviation == RotationalSpeedUnits.RevolutionPerSecond:
            return """r/s"""
        
        if unit_abbreviation == RotationalSpeedUnits.RevolutionPerMinute:
            return """rpm"""
        
        if unit_abbreviation == RotationalSpeedUnits.NanoradianPerSecond:
            return """nrad/s"""
        
        if unit_abbreviation == RotationalSpeedUnits.MicroradianPerSecond:
            return """μrad/s"""
        
        if unit_abbreviation == RotationalSpeedUnits.MilliradianPerSecond:
            return """mrad/s"""
        
        if unit_abbreviation == RotationalSpeedUnits.CentiradianPerSecond:
            return """crad/s"""
        
        if unit_abbreviation == RotationalSpeedUnits.DeciradianPerSecond:
            return """drad/s"""
        
        if unit_abbreviation == RotationalSpeedUnits.NanodegreePerSecond:
            return """n°/s"""
        
        if unit_abbreviation == RotationalSpeedUnits.MicrodegreePerSecond:
            return """μ°/s"""
        
        if unit_abbreviation == RotationalSpeedUnits.MillidegreePerSecond:
            return """m°/s"""
        